import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import permission_required
from django.shortcuts import get_object_or_404
from django.shortcuts import render, redirect
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from .form import OpForm

from .models import OpTimer

logger = logging.getLogger(__name__)


@login_required
@permission_required('auth.optimer_view')
def optimer_view(request):
    logger.debug("optimer_view called by user %s" % request.user)
    base_query = OpTimer.objects.select_related('eve_character')
    render_items = {'optimer': base_query.all(),
                    'future_timers': base_query.filter(
                        start__gte=timezone.now()),
                    'past_timers': base_query.filter(
                        start__lt=timezone.now()).order_by('-start')}

    return render(request, 'optimer/management.html', context=render_items)


@login_required
@permission_required('auth.optimer_management')
def add_optimer_view(request):
    logger.debug("add_optimer_view called by user %s" % request.user)
    if request.method == 'POST':
        form = OpForm(request.POST)
        logger.debug("Request type POST contains form valid: %s" % form.is_valid())
        if form.is_valid():
            # Get Current Time
            post_time = timezone.now()
            # Get character
            character = request.user.profile.main_character
            # handle valid form
            op = OpTimer()
            op.doctrine = form.cleaned_data['doctrine']
            op.system = form.cleaned_data['system']
            op.start = form.cleaned_data['start']
            op.duration = form.cleaned_data['duration']
            op.operation_name = form.cleaned_data['operation_name']
            op.fc = form.cleaned_data['fc']
            op.create_time = post_time
            op.eve_character = character
            op.save()
            logger.info("User %s created op timer with name %s" % (request.user, op.operation_name))
            messages.success(request, _('Created operation timer for %(opname)s.') % {"opname": op.operation_name})
            return redirect("optimer:view")
    else:
        logger.debug("Returning new opForm")
        form = OpForm()

    render_items = {'form': form}

    return render(request, 'optimer/add.html', context=render_items)


@login_required
@permission_required('auth.optimer_management')
def remove_optimer(request, optimer_id):
    logger.debug("remove_optimer called by user %s for operation id %s" % (request.user, optimer_id))
    op = get_object_or_404(OpTimer, id=optimer_id)
    op.delete()
    logger.info("Deleting optimer id %s by user %s" % (optimer_id, request.user))
    messages.success(request, _('Removed operation timer for %(opname)s.') % {"opname": op.operation_name})
    return redirect("optimer:view")


@login_required
@permission_required('auth.optimer_management')
def edit_optimer(request, optimer_id):
    logger.debug("edit_optimer called by user %s for optimer id %s" % (request.user, optimer_id))
    op = get_object_or_404(OpTimer, id=optimer_id)
    if request.method == 'POST':
        form = OpForm(request.POST)
        logger.debug("Received POST request containing update optimer form, is valid: %s" % form.is_valid())
        if form.is_valid():
            character = request.user.profile.main_character
            op.doctrine = form.cleaned_data['doctrine']
            op.system = form.cleaned_data['system']
            op.start = form.cleaned_data['start']
            op.duration = form.cleaned_data['duration']
            op.operation_name = form.cleaned_data['operation_name']
            op.fc = form.cleaned_data['fc']
            op.eve_character = character
            logger.info("User %s updating optimer id %s " % (request.user, optimer_id))
            op.save()
            messages.success(request, _('Saved changes to operation timer for %(opname)s.') % {"opname": op.operation_name})
            return redirect("optimer:view")
    else:
        data = {
            'doctrine': op.doctrine,
            'system': op.system,
            'start': op.start,
            'duration': op.duration,
            'operation_name': op.operation_name,
            'fc': op.fc,
        }
        form = OpForm(initial=data)
    return render(request, 'optimer/update.html', context={'form': form})



from django.contrib import admin
from .models import AuthTS, Teamspeak3User, StateGroup


class Teamspeak3UserAdmin(admin.ModelAdmin):
        list_display = ('user', 'uid', 'perm_key')
        search_fields = ('user__username', 'uid', 'perm_key')


class AuthTSgroupAdmin(admin.ModelAdmin):
    fields = ['auth_group', 'ts_group']
    filter_horizontal = ('ts_group',)


@admin.register(StateGroup)
class StateGroupAdmin(admin.ModelAdmin):
    list_display = ('state', 'ts_group')
    search_fields = ('state__name', 'ts_group__ts_group_name')


admin.site.register(AuthTS, AuthTSgroupAdmin)
admin.site.register(Teamspeak3User, Teamspeak3UserAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-08 02:54
from __future__ import unicode_literals

from django.db import migrations


def delete_permissions(apps, schema_editor):
    HRApplication = apps.get_model('hrapplications', 'HRApplication')
    HRApplicationComment = apps.get_model('hrapplications', 'HRApplicationComment')
    ContentType = apps.get_model('contenttypes', 'ContentType')
    Permission = apps.get_model('auth', 'Permission')
    ct1 = ContentType.objects.get_for_model(HRApplication)
    ct2 = ContentType.objects.get_for_model(HRApplicationComment)
    Permission.objects.filter(content_type__in=[ct1, ct2]).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('hrapplications', '0005_sorted_questions'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='hrapplication',
            name='corp',
        ),
        migrations.RemoveField(
            model_name='hrapplication',
            name='reviewer_character',
        ),
        migrations.RemoveField(
            model_name='hrapplication',
            name='reviewer_inprogress_character',
        ),
        migrations.RemoveField(
            model_name='hrapplication',
            name='reviewer_user',
        ),
        migrations.RemoveField(
            model_name='hrapplication',
            name='user',
        ),
        migrations.RemoveField(
            model_name='hrapplicationcomment',
            name='application',
        ),
        migrations.RemoveField(
            model_name='hrapplicationcomment',
            name='commenter_character',
        ),
        migrations.RemoveField(
            model_name='hrapplicationcomment',
            name='commenter_user',
        ),
        migrations.RunPython(delete_permissions, migrations.RunPython.noop),
        migrations.DeleteModel(
            name='HRApplication',
        ),
        migrations.DeleteModel(
            name='HRApplicationComment',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-11 23:14
from __future__ import unicode_literals

from django.db import migrations

import logging

logger = logging.getLogger(__name__)


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0012_remove_add_delete_authservicesinfo_permissions'),
    ]

    operations = [
        # Remove fields
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='discord_uid',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='discourse_enabled',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='forum_username',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='ipboard_username',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='ips4_id',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='ips4_username',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='jabber_username',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='market_username',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='mumble_username',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='smf_username',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='teamspeak3_perm_key',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='teamspeak3_uid',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='xenforo_username',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-22 23:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('timerboard', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timer',
            name='details',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='timer',
            name='objective',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='timer',
            name='planet_moon',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='timer',
            name='structure',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='timer',
            name='system',
            field=models.CharField(default='', max_length=254),
        ),
    ]



from django.contrib import admin
from .models import SmfUser


class SmfUserAdmin(admin.ModelAdmin):
        list_display = ('user', 'username')
        search_fields = ('user__username', 'username')

admin.site.register(SmfUser, SmfUserAdmin)



from django import forms
from django.utils.translation import ugettext_lazy as _


class JabberBroadcastForm(forms.Form):
    group = forms.ChoiceField(label=_('Group'), widget=forms.Select)
    message = forms.CharField(label=_('Message'), widget=forms.Textarea)



from django.conf.urls import url

from . import views

app_name = 'optimer'

urlpatterns = [
    url(r'^$', views.optimer_view, name='view'),
    url(r'^add$', views.add_optimer_view, name='add'),
    url(r'^(\w+)/remove$', views.remove_optimer, name='remove'),
    url(r'^(\w+)/edit$', views.edit_optimer, name='edit'),
    ]



from django.conf.urls import url

from . import views

app_name = 'srp'

urlpatterns = [
    # SRP URLS
    url(r'^$', views.srp_management, name='management'),
    url(r'^all/$', views.srp_management, {'all': True}, name='all'),
    url(r'^(\w+)/view$', views.srp_fleet_view, name='fleet'),
    url(r'^add/$', views.srp_fleet_add_view, name='add'),
    url(r'^(\w+)/edit$', views.srp_fleet_edit_view, name='edit'),
    url(r'^(\w+)/request', views.srp_request_view, name='request'),

    # SRP URLS
    url(r'^(\w+)/remove$', views.srp_fleet_remove, name='remove'),
    url(r'^(\w+)/disable$', views.srp_fleet_disable, name='disable'),
    url(r'^(\w+)/enable$', views.srp_fleet_enable, name='enable'),
    url(r'^(\w+)/complete$', views.srp_fleet_mark_completed,
        name='mark_completed'),
    url(r'^(\w+)/incomplete$', views.srp_fleet_mark_uncompleted,
        name='mark_uncompleted'),
    url(r'^request/remove/', views.srp_request_remove,
        name="request_remove"),
    url(r'^request/approve/', views.srp_request_approve,
        name='request_approve'),
    url(r'^request/reject/', views.srp_request_reject,
        name='request_reject'),
    url(r'^request/(\w+)/update', views.srp_request_update_amount,
        name="request_update_amount"),
]



from django.conf.urls import url, include

from . import views

app_name = 'ips4'

module_urls = [
    # IPS4 Service Control
    url(r'^activate/$', views.activate_ips4, name='activate'),
    url(r'^deactivate/$', views.deactivate_ips4, name='deactivate'),
    url(r'^reset_password/$', views.reset_ips4_password, name='reset_password'),
    url(r'^set_password/$', views.set_ips4_password, name='set_password'),
]

urlpatterns = [
    url(r'^ips4/', include((module_urls, app_name), namespace=app_name))
]



import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, redirect

from allianceauth.services.forms import ServicePasswordForm
from .manager import Ips4Manager
from .models import Ips4User
from .tasks import Ips4Tasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'ips4.access_ips4'


@login_required
@permission_required(ACCESS_PERM)
def activate_ips4(request):
    logger.debug("activate_ips4 called by user %s" % request.user)
    character = request.user.profile.main_character
    logger.debug("Adding IPS4 user for user %s with main character %s" % (request.user, character))
    result = Ips4Manager.add_user(Ips4Tasks.get_username(request.user), request.user.email)
    # if empty we failed
    if result[0] != "" and not Ips4Tasks.has_account(request.user):
        ips_user = Ips4User.objects.create(user=request.user, id=result[2], username=result[0])
        logger.debug("Updated authserviceinfo for user %s with IPS4 credentials." % request.user)
        # update_ips4_groups.delay(request.user.pk)
        logger.info("Successfully activated IPS4 for user %s" % request.user)
        messages.success(request, 'Activated IPSuite4 account.')
        credentials = {
            'username': result[0],
            'password': result[1],
        }
        return render(request, 'services/service_credentials.html',
                      context={'credentials': credentials, 'service': 'IPSuite4'})
    else:
        logger.error("Unsuccessful attempt to activate IPS4 for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your IPSuite4 account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_ips4_password(request):
    logger.debug("reset_ips4_password called by user %s" % request.user)
    if Ips4Tasks.has_account(request.user):
        result = Ips4Manager.update_user_password(request.user.ips4.username)
        # false we failed
        if result != "":
            logger.info("Successfully reset IPS4 password for user %s" % request.user)
            messages.success(request, 'Reset IPSuite4 password.')
            credentials = {
                'username': request.user.ips4.username,
                'password': result,
            }
            return render(request, 'services/service_credentials.html',
                          context={'credentials': credentials, 'service': 'IPSuite4'})

    logger.error("Unsuccessful attempt to reset IPS4 password for user %s" % request.user)
    messages.error(request, 'An error occurred while processing your IPSuite4 account.')
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def set_ips4_password(request):
    logger.debug("set_ips4_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        if form.is_valid() and Ips4Tasks.has_account(request.user):
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            result = Ips4Manager.update_custom_password(request.user.ips4.username, plain_password=password)
            if result != "":
                logger.info("Successfully set IPS4 password for user %s" % request.user)
                messages.success(request, 'Set IPSuite4 password.')
            else:
                logger.error("Failed to install custom IPS4 password for user %s" % request.user)
                messages.error(request, 'An error occurred while processing your IPSuite4 account.')
            return redirect('services:services')
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'IPS4'}
    return render(request, 'services/service_password.html', context=context)


@login_required
@permission_required(ACCESS_PERM)
def deactivate_ips4(request):
    logger.debug("deactivate_ips4 called by user %s" % request.user)
    if Ips4Tasks.delete_user(request.user):
        logger.info("Successfully deactivated IPS4 for user %s" % request.user)
        messages.success(request, 'Deactivated IPSuite4 account.')
    else:
        logger.error("Unsuccessful attempt to deactivate IPS4 for user %s" % request.user)
        messages.error(request, 'An error occurred while processing your IPSuite4 account.')
    return redirect("services:services")




import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render, redirect
from django.utils.translation import ugettext_lazy as _

from allianceauth.services.forms import ServicePasswordForm
from .manager import SeatManager
from .models import SeatUser
from .tasks import SeatTasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'seat.access_seat'
SERVICE_NAME = {'service': 'SeAT'}


@login_required
@permission_required(ACCESS_PERM)
def activate_seat(request):
    logger.debug("activate_seat called by user %s" % request.user)
    character = request.user.profile.main_character
    logger.debug("Checking SeAT for inactive users with the same username")
    stat = SeatManager.check_user_status(character.character_name)
    if stat == {}:
        logger.debug("User not found, adding SeAT user for user %s with main character %s" % (request.user, character))
        result = SeatManager.add_user(SeatTasks.get_username(request.user), request.user.email)
    else:
        logger.debug("User found, resetting password")
        username = SeatManager.enable_user(stat["name"])
        password = SeatManager.update_user_password(username, request.user.email)
        result = [username, password]
    # if empty we failed
    if result[0] and result[1]:
        SeatUser.objects.update_or_create(user=request.user, defaults={'username': result[0]})
        logger.debug("Updated SeatUser for user %s with SeAT credentials. Adding eve-apis..." % request.user)
        SeatTasks.update_roles.delay(request.user.pk)
        logger.info("Successfully activated SeAT for user %s" % request.user)
        messages.add_message(request, messages.SUCCESS, _('Successfully activated your %(service)s account.') %
                             SERVICE_NAME)
        credentials = {
            'username': request.user.seat.username,
            'password': result[1],
        }
        return render(request, 'services/service_credentials.html',
                      context={'credentials': credentials, 'service': 'SeAT'})
    messages.add_message(request, messages.ERROR,
                         _('Failed to activate your %(service)s account, please contact your administrator.') %
                         SERVICE_NAME)
    logger.error("Unsuccessful attempt to activate seat for user %s" % request.user)
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def deactivate_seat(request):
    logger.debug("deactivate_seat called by user %s" % request.user)
    # false we failed
    if SeatTasks.delete_user(request.user):
        messages.add_message(request, messages.SUCCESS,
                             _('Successfully deactivated your %(service)s account.') % SERVICE_NAME)
        logger.info("Successfully deactivated SeAT for user %s" % request.user)
        return redirect("services:services")
    else:
        logging.error("User does not have a SeAT account")
    messages.add_message(request, messages.ERROR,
                         _('Failed to deactivate your %(service)s account, please contact your administrator.') %
                         SERVICE_NAME)
    logger.error("Unsuccessful attempt to activate SeAT for user %s" % request.user)
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_seat_password(request):
    logger.debug("reset_seat_password called by user %s" % request.user)
    if SeatTasks.has_account(request.user):
        result = SeatManager.update_user_password(request.user.seat.username, request.user.email)
        # false we failed
        if result:
            credentials = {
                'username': request.user.seat.username,
                'password': result,
            }
            messages.add_message(request, messages.SUCCESS,
                                 _('Successfully reset your %(service)s password.') % {'service': 'SeAT'})
            logger.info("Succesfully reset SeAT password for user %s" % request.user)
            return render(request, 'services/service_credentials.html',
                          context={'credentials': credentials, 'service': 'SeAT'})
    logger.error("Unsuccessful attempt to reset SeAT password for user %s" % request.user)
    messages.add_message(request, messages.ERROR,
                         _('Failed to reset your %(service)s password, please contact your administrator.') %
                         {'service': 'SeAT'})
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def set_seat_password(request):
    logger.debug("set_seat_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        if form.is_valid() and SeatTasks.has_account(request.user):
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            result = SeatManager.update_user_password(request.user.seat.username,
                                                      request.user.email,
                                                      plain_password=password)
            if result:
                messages.add_message(request, messages.SUCCESS,
                                     _('Successfully set your %(service)s password.') % SERVICE_NAME)
                logger.info("Succesfully reset SeAT password for user %s" % request.user)
                return redirect("services:services")
            else:
                messages.add_message(request, messages.ERROR,
                                     _('Failed to set your %(service)s password, please contact your administrator.') %
                                     SERVICE_NAME)
                logger.error("Failed to install custom SeAT password for user %s" % request.user)
        else:
            messages.add_message(request, messages.ERROR, _('Invalid password.'))
            logger.error("Invalid SeAT password provided")
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()
    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'SeAT'}
    return render(request, 'services/service_password.html', context)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-05 21:39
from __future__ import unicode_literals

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models
from django.utils import timezone

import allianceauth.fleetactivitytracking.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('eveonline', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Fat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('system', models.CharField(max_length=30)),
                ('shiptype', models.CharField(max_length=30)),
                ('station', models.CharField(max_length=125)),
                ('character', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveCharacter')),
            ],
        ),
        migrations.CreateModel(
            name='Fatlink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('fatdatetime', models.DateTimeField(default=timezone.now)),
                ('duration', models.PositiveIntegerField()),
                ('fleet', models.CharField(default='', max_length=254)),
                ('name', models.CharField(max_length=254)),
                ('hash', models.CharField(max_length=254, unique=True)),
                ('creator', models.ForeignKey(on_delete=models.SET(
                    allianceauth.fleetactivitytracking.models.get_sentinel_user), to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='fat',
            name='fatlink',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='fleetactivitytracking.Fatlink'),
        ),
        migrations.AddField(
            model_name='fat',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='fat',
            unique_together=set([('character', 'fatlink')]),
        ),
    ]



"""
Copyright (c) 2014 Torchbox Ltd and individual contributors.
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.

    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

    3. Neither the name of Torchbox nor the names of its contributors may be used
       to endorse or promote products derived from this software without
       specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Based on https://github.com/torchbox/wagtail/blob/master/wagtail/wagtailcore/hooks.py
"""

from importlib import import_module

from django.apps import apps
from django.utils.module_loading import module_has_submodule

import logging

logger = logging.getLogger(__name__)

_hooks = {}  # Dict of Name: Fn's of registered hooks

_all_hooks_registered = False  # If all hooks have been searched for and registered yet


def register(name, fn=None):
    """
    Decorator to register a function as a hook

    Register hook for ``hook_name``. Can be used as a decorator::
        @register('hook_name')
        def my_hook(...):
            pass

    or as a function call::
        def my_hook(...):
            pass
        register('hook_name', my_hook)

    :param name: str Name of the hook/callback to register it as
    :param fn: function to register in the hook/callback
    :return: function Decorator if applied as a decorator
    """
    def _hook_add(func):
        if name not in _hooks:
            logger.debug("Creating new hook %s" % name)
            _hooks[name] = []

        logger.debug('Registering hook %s for function %s' % (name, fn))
        _hooks[name].append(func)

    if fn is None:
        # Behave like a decorator
        def decorator(func):
            _hook_add(func)
            return func
        return decorator
    else:
        # Behave like a function, just register hook
        _hook_add(fn)


def get_app_modules():
    """
    Get all the modules of the django app
    :return: name, module tuple
    """
    for app in apps.get_app_configs():
        yield app.name, app.module


def get_app_submodules(module_name):
    """
    Get a specific sub module of the app
    :param module_name: module name to get
    :return: name, module tuple
    """
    for name, module in get_app_modules():
        if module_has_submodule(module, module_name):
            yield name, import_module('{0}.{1}'.format(name, module_name))


def register_all_hooks():
    """
    Register all hooks found in 'auth_hooks' sub modules
    :return:
    """
    global _all_hooks_registered
    if not _all_hooks_registered:
        logger.debug("Searching for hooks")
        hooks = list(get_app_submodules('auth_hooks'))
        logger.debug("Got %s hooks" % len(hooks))
        _all_hooks_registered = True


def get_hooks(name):
    """
    Get all the hook functions for the given hook name
    :param name: str name of the hook to get the functions for
    :return: list of hook functions
    """
    register_all_hooks()
    return _hooks.get(name, [])




import logging

from django.template.loader import render_to_string
from django.conf import settings

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook
from .tasks import Teamspeak3Tasks
from .urls import urlpatterns

logger = logging.getLogger(__name__)


class Teamspeak3Service(ServicesHook):
    def __init__(self):
        ServicesHook.__init__(self)
        self.name = 'teamspeak3'
        self.urlpatterns = urlpatterns
        self.service_ctrl_template = 'services/teamspeak3/teamspeak3_service_ctrl.html'
        self.access_perm = 'teamspeak3.access_teamspeak3'
        self.name_format = '[{corp_ticker}]{character_name}'

    def delete_user(self, user, notify_user=False):
        logger.debug('Deleting user %s %s account' % (user, self.name))
        return Teamspeak3Tasks.delete_user(user, notify_user=notify_user)

    def update_groups(self, user):
        logger.debug('Updating %s groups for %s' % (self.name, user))
        Teamspeak3Tasks.update_groups.delay(user.pk)

    def validate_user(self, user):
        logger.debug('Validating user %s %s account' % (user, self.name))
        if Teamspeak3Tasks.has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user, notify_user=True)

    def update_all_groups(self):
        logger.debug('Update all %s groups called' % self.name)
        Teamspeak3Tasks.update_all_groups.delay()

    def service_active_for_user(self, user):
        return user.has_perm(self.access_perm)

    def render_services_ctrl(self, request):
        authinfo = {'teamspeak3_uid': '',
                    'teamspeak3_perm_key': '',}
        if Teamspeak3Tasks.has_account(request.user):
            authinfo['teamspeak3_uid'] = request.user.teamspeak3.uid
            authinfo['teamspeak3_perm_key'] = request.user.teamspeak3.perm_key

        return render_to_string(self.service_ctrl_template, {
            'authinfo': authinfo, 'TEAMSPEAK3_PUBLIC_URL': getattr(settings, 'TEAMSPEAK3_PUBLIC_URL', ''),
        }, request=request)


@hooks.register('services_hook')
def register_service():
    return Teamspeak3Service()



from django.db import models
import logging

logger = logging.getLogger(__name__)


class CorpStatsQuerySet(models.QuerySet):
    def visible_to(self, user):
        # superusers get all visible
        if user.is_superuser:
            logger.debug('Returning all corpstats for superuser %s.' % user)
            return self

        try:
            char = user.profile.main_character
            assert char
            # build all accepted queries
            queries = [models.Q(token__user=user)]
            if user.has_perm('corputils.view_alliance_corpstats'):
                if char.alliance_id is not None:
                    queries.append(models.Q(corp__alliance__alliance_id=char.alliance_id))
                else:
                    queries.append(models.Q(corp__corporation_id=char.corporation_id))
            if user.has_perm('corputils.view_corp_corpstats'):
                if user.has_perm('corputils.view_alliance_corpstats'):
                    pass
                else:
                    queries.append(models.Q(corp__corporation_id=char.corporation_id))
            if user.has_perm('corputils.view_state_corpstats'):
                queries.append(models.Q(corp__in=user.profile.state.member_corporations.all()))
                queries.append(models.Q(corp__alliance__in=user.profile.state.member_alliances.all()))
            logger.debug('%s queries for user %s visible corpstats.' % (len(queries), user))
            # filter based on queries
            query = queries.pop()
            for q in queries:
                query |= q
            return self.filter(query)
        except AssertionError:
            logger.debug('User %s has no main character. No corpstats visible.' % user)
            return self.none()        


class CorpStatsManager(models.Manager):
    def get_queryset(self):
        return CorpStatsQuerySet(self.model, using=self._db)

    def visible_to(self, user):
        return self.get_queryset().visible_to(user)



# This will make sure the app is always imported when
# Django starts so that shared_task will use this app.

__version__ = '2.1.0'
NAME = 'Alliance Auth v%s' % __version__
default_app_config = 'allianceauth.apps.AllianceAuthConfig'



from django.apps import AppConfig
from django.core.checks import register, Tags


class AuthenticationConfig(AppConfig):
    name = 'allianceauth.authentication'
    label = 'authentication'

    def ready(self):
        super(AuthenticationConfig, self).ready()
        from allianceauth.authentication import checks, signals
        register(Tags.security)(checks.check_login_scopes_setting)



from unittest import mock

from django.test import TestCase, RequestFactory
from django import urls
from django.contrib.auth.models import User, Group, Permission
from django.core.exceptions import ObjectDoesNotExist

from allianceauth.tests.auth_utils import AuthUtils

from .auth_hooks import OpenfireService
from .models import OpenfireUser
from .tasks import OpenfireTasks

MODULE_PATH = 'allianceauth.services.modules.openfire'
DEFAULT_AUTH_GROUP = 'Member'


def add_permissions():
    permission = Permission.objects.get(codename='access_openfire')
    members = Group.objects.get_or_create(name=DEFAULT_AUTH_GROUP)[0]
    AuthUtils.add_permissions_to_groups([permission], [members])


class OpenfireHooksTestCase(TestCase):
    def setUp(self):
        self.member = 'member_user'
        member = AuthUtils.create_member(self.member)
        OpenfireUser.objects.create(user=member, username=self.member)
        self.none_user = 'none_user'
        none_user = AuthUtils.create_user(self.none_user)
        self.service = OpenfireService
        add_permissions()

    def test_has_account(self):
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)
        self.assertTrue(OpenfireTasks.has_account(member))
        self.assertFalse(OpenfireTasks.has_account(none_user))

    def test_service_enabled(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)

        self.assertTrue(service.service_active_for_user(member))
        self.assertFalse(service.service_active_for_user(none_user))

    @mock.patch(MODULE_PATH + '.tasks.OpenfireManager')
    def test_update_all_groups(self, manager):
        service = self.service()
        service.update_all_groups()
        # Check member and blue user have groups updated
        self.assertTrue(manager.update_user_groups.called)
        self.assertEqual(manager.update_user_groups.call_count, 1)

    def test_update_groups(self):
        # Check member has Member group updated
        with mock.patch(MODULE_PATH + '.tasks.OpenfireManager') as manager:
            service = self.service()
            member = User.objects.get(username=self.member)
            service.update_groups(member)
            self.assertTrue(manager.update_user_groups.called)
            args, kwargs = manager.update_user_groups.call_args
            user_id, groups = args
            self.assertIn(DEFAULT_AUTH_GROUP, groups)
            self.assertEqual(user_id, member.openfire.username)

        # Check none user does not have groups updated
        with mock.patch(MODULE_PATH + '.tasks.OpenfireManager') as manager:
            service = self.service()
            none_user = User.objects.get(username=self.none_user)
            service.update_groups(none_user)
            self.assertFalse(manager.update_user_groups.called)

    @mock.patch(MODULE_PATH + '.tasks.OpenfireManager')
    def test_validate_user(self, manager):
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.member)
        service.validate_user(member)
        self.assertTrue(member.openfire)

        # Test none user is deleted
        none_user = User.objects.get(username=self.none_user)
        OpenfireUser.objects.create(user=none_user, username='abc123')
        service.validate_user(none_user)
        self.assertTrue(manager.delete_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            none_openfire = User.objects.get(username=self.none_user).openfire

    @mock.patch(MODULE_PATH + '.tasks.OpenfireManager')
    def test_delete_user(self, manager):
        member = User.objects.get(username=self.member)

        service = self.service()
        result = service.delete_user(member)

        self.assertTrue(result)
        self.assertTrue(manager.delete_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            openfire_user = User.objects.get(username=self.member).openfire

    def test_render_services_ctrl(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        request = RequestFactory().get('/services/')
        request.user = member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn(urls.reverse('openfire:deactivate'), response)
        self.assertIn(urls.reverse('openfire:reset_password'), response)
        self.assertIn(urls.reverse('openfire:set_password'), response)

        # Test register becomes available
        member.openfire.delete()
        member = User.objects.get(username=self.member)
        request.user = member
        response = service.render_services_ctrl(request)
        self.assertIn(urls.reverse('openfire:activate'), response)


class OpenfireViewsTestCase(TestCase):
    def setUp(self):
        self.member = AuthUtils.create_member('auth_member')
        self.member.email = 'auth_member@example.com'
        self.member.save()
        AuthUtils.add_main_character(self.member, 'auth_member', '12345', corp_id='111', corp_name='Test Corporation')
        add_permissions()

    def login(self):
        self.client.force_login(self.member)

    @mock.patch(MODULE_PATH + '.tasks.OpenfireManager')
    @mock.patch(MODULE_PATH + '.views.OpenfireManager')
    def test_activate(self, manager, tasks_manager):
        self.login()
        expected_username = 'auth_member'
        manager.add_user.return_value = (expected_username, 'abc123')

        response = self.client.get(urls.reverse('openfire:activate'))

        self.assertTrue(manager.add_user.called)
        self.assertTrue(tasks_manager.update_user_groups.called)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed('services/service_credentials.html')
        self.assertContains(response, expected_username)
        openfire_user = OpenfireUser.objects.get(user=self.member)
        self.assertEqual(openfire_user.username, expected_username)

    @mock.patch(MODULE_PATH + '.tasks.OpenfireManager')
    def test_deactivate(self, manager):
        self.login()
        OpenfireUser.objects.create(user=self.member, username='some member')

        response = self.client.get(urls.reverse('openfire:deactivate'))

        self.assertTrue(manager.delete_user.called)
        self.assertRedirects(response, expected_url=urls.reverse('services:services'), target_status_code=200)
        with self.assertRaises(ObjectDoesNotExist):
            openfire_user = User.objects.get(pk=self.member.pk).openfire

    @mock.patch(MODULE_PATH + '.views.OpenfireManager')
    def test_set_password(self, manager):
        self.login()
        OpenfireUser.objects.create(user=self.member, username='some member')

        response = self.client.post(urls.reverse('openfire:set_password'), data={'password': '1234asdf'})

        self.assertTrue(manager.update_user_pass.called)
        args, kwargs = manager.update_user_pass.call_args
        self.assertEqual(kwargs['password'], '1234asdf')
        self.assertRedirects(response, expected_url=urls.reverse('services:services'), target_status_code=200)

    @mock.patch(MODULE_PATH + '.views.OpenfireManager')
    def test_reset_password(self, manager):
        self.login()
        OpenfireUser.objects.create(user=self.member, username='some member')

        manager.update_user_pass.return_value = 'hunter2'

        response = self.client.get(urls.reverse('openfire:reset_password'))

        self.assertTemplateUsed(response, 'services/service_credentials.html')
        self.assertContains(response, 'some member')
        self.assertContains(response, 'hunter2')


class OpenfireManagerTestCase(TestCase):
    def setUp(self):
        from .manager import OpenfireManager
        self.manager = OpenfireManager

    def test_generate_random_password(self):
        password = self.manager._OpenfireManager__generate_random_pass()

        self.assertEqual(len(password), 16)
        self.assertIsInstance(password, type(''))

    def test__sanitize_username(self):
        test_username = " My_Test User\"'&/:<>@name\\20name"

        result_username = self.manager._OpenfireManager__sanitize_username(test_username)

        self.assertEqual(result_username, 'My_Test\\20User\\22\\27\\26\\2f\\3a\\3c\\3e\\40name\\5c20name')

    def test__sanitize_groupname(self):
        test_groupname = " My_Test Groupname"

        result_groupname = self.manager._sanitize_groupname(test_groupname)

        self.assertEqual(result_groupname, "my_testgroupname")

    @mock.patch(MODULE_PATH + '.manager.ofUsers')
    def test_update_user_groups(self, api):
        groups = ["AddGroup", "othergroup", "Guest Group"]
        server_groups = ["othergroup", "Guest Group", "REMOVE group"]
        username = "testuser"
        api_instance = api.return_value
        api_instance.get_user_groups.return_value = {'groupname': server_groups}

        self.manager.update_user_groups(username, groups)

        self.assertTrue(api_instance.add_user_groups.called)
        args, kwargs = api_instance.add_user_groups.call_args
        self.assertEqual(args[1], ["addgroup"])

        self.assertTrue(api_instance.delete_user_groups.called)
        args, kwargs = api_instance.delete_user_groups.call_args
        self.assertEqual(args[1], ["removegroup"])



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-16 01:35
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('services', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='discordauthtoken',
            name='user',
        ),
        migrations.DeleteModel(
            name='DiscordAuthToken',
        ),
    ]



from django.conf.urls import url

from . import views

app_name = 'hrapplications'

urlpatterns = [
    url(r'^$', views.hr_application_management_view,
        name="index"),
    url(r'^create/$', views.hr_application_create_view,
        name="create_view"),
    url(r'^create/(\d+)', views.hr_application_create_view,
        name="create_view"),
    url(r'^remove/(\w+)', views.hr_application_remove,
        name="remove"),
    url(r'^view/(\w+)', views.hr_application_view,
        name="view"),
    url(r'^personal/view/(\w+)', views.hr_application_personal_view,
        name="personal_view"),
    url(r'^personal/removal/(\w+)',
        views.hr_application_personal_removal,
        name="personal_removal"),
    url(r'^approve/(\w+)', views.hr_application_approve,
        name="approve"),
    url(r'^reject/(\w+)', views.hr_application_reject,
        name="reject"),
    url(r'^search/', views.hr_application_search,
        name="search"),
    url(r'^mark_in_progress/(\w+)', views.hr_application_mark_in_progress,
        name="mark_in_progress"),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-22 23:35
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corputils', '0002_migrate_permissions'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='corpstats',
            options={'permissions': (('corp_apis', 'Can view API keys of members of their corporation.'), ('alliance_apis', 'Can view API keys of members of their alliance.'), ('blue_apis', 'Can view API keys of members of blue corporations.'), ('view_corp_corpstats', 'Can view corp stats of their corporation.'), ('view_alliance_corpstats', 'Can view corp stats of members of their alliance.'), ('view_blue_corpstats', 'Can view corp stats of blue corporations.')), 'verbose_name': 'corp stats', 'verbose_name_plural': 'corp stats'},
        ),
    ]



from unittest import mock

from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils
from .models import CorpStats, CorpMember
from allianceauth.eveonline.models import EveCorporationInfo, EveAllianceInfo, EveCharacter
from esi.models import Token
from esi.errors import TokenError
from bravado.exception import HTTPForbidden
from django.contrib.auth.models import Permission
from allianceauth.authentication.models import CharacterOwnership


class CorpStatsManagerTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test')
        AuthUtils.add_main_character(cls.user, 'test character', '1', corp_id='2', corp_name='test_corp', corp_ticker='TEST', alliance_id='3', alliance_name='TEST')
        cls.user.profile.refresh_from_db()
        cls.alliance = EveAllianceInfo.objects.create(alliance_id='3', alliance_name='test alliance', alliance_ticker='TEST', executor_corp_id='2')
        cls.corp = EveCorporationInfo.objects.create(corporation_id='2', corporation_name='test corp', corporation_ticker='TEST', alliance=cls.alliance, member_count=1)
        cls.token = Token.objects.create(user=cls.user, access_token='a', character_id='1', character_name='test character', character_owner_hash='z')
        cls.corpstats = CorpStats.objects.create(corp=cls.corp, token=cls.token)
        cls.view_corp_permission = Permission.objects.get_by_natural_key('view_corp_corpstats', 'corputils', 'corpstats')
        cls.view_alliance_permission = Permission.objects.get_by_natural_key('view_alliance_corpstats', 'corputils', 'corpstats')
        cls.view_state_permission = Permission.objects.get_by_natural_key('view_state_corpstats', 'corputils', 'corpstats')
        cls.state = AuthUtils.create_state('test state', 500)
        AuthUtils.assign_state(cls.user, cls.state, disconnect_signals=True)

    def setUp(self):
        self.user.refresh_from_db()
        self.user.user_permissions.clear()
        self.state.refresh_from_db()
        self.state.member_corporations.clear()
        self.state.member_alliances.clear()

    def test_visible_superuser(self):
        self.user.is_superuser = True
        cs = CorpStats.objects.visible_to(self.user)
        self.assertIn(self.corpstats, cs)

    def test_visible_corporation(self):
        self.user.user_permissions.add(self.view_corp_permission)
        cs = CorpStats.objects.visible_to(self.user)
        self.assertIn(self.corpstats, cs)

    def test_visible_alliance(self):
        self.user.user_permissions.add(self.view_alliance_permission)
        cs = CorpStats.objects.visible_to(self.user)
        self.assertIn(self.corpstats, cs)

    def test_visible_state_corp_member(self):
        self.state.member_corporations.add(self.corp)
        self.user.user_permissions.add(self.view_state_permission)
        cs = CorpStats.objects.visible_to(self.user)
        self.assertIn(self.corpstats, cs)

    def test_visible_state_alliance_member(self):
        self.state.member_alliances.add(self.alliance)
        self.user.user_permissions.add(self.view_state_permission)
        cs = CorpStats.objects.visible_to(self.user)
        self.assertIn(self.corpstats, cs)


class CorpStatsUpdateTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test')
        AuthUtils.add_main_character(cls.user, 'test character', '1', corp_id='2', corp_name='test_corp', corp_ticker='TEST', alliance_id='3', alliance_name='TEST')
        cls.token = Token.objects.create(user=cls.user, access_token='a', character_id='1', character_name='test character', character_owner_hash='z')
        cls.corp = EveCorporationInfo.objects.create(corporation_id='2', corporation_name='test corp', corporation_ticker='TEST', member_count=1)

    def setUp(self):
        self.corpstats = CorpStats.objects.get_or_create(token=self.token, corp=self.corp)[0]

    def test_can_update(self):
        self.assertTrue(self.corpstats.can_update(self.user))
        self.corpstats.token.user = None
        self.assertFalse(self.corpstats.can_update(self.user))
        self.user.is_superuser = True
        self.assertTrue(self.corpstats.can_update(self.user))
        self.user.refresh_from_db()
        self.corpstats.token.refresh_from_db()

    @mock.patch('esi.clients.SwaggerClient')
    def test_update_add_member(self, SwaggerClient):
        SwaggerClient.from_spec.return_value.Character.get_characters_character_id.return_value.result.return_value = {'corporation_id': 2}
        SwaggerClient.from_spec.return_value.Corporation.get_corporations_corporation_id_members.return_value.result.return_value = [1]
        SwaggerClient.from_spec.return_value.Universe.post_universe_names.return_value.result.return_value = [{'id': 1, 'name': 'test character'}]
        self.corpstats.update()
        self.assertTrue(CorpMember.objects.filter(character_id='1', character_name='test character', corpstats=self.corpstats).exists())

    @mock.patch('esi.clients.SwaggerClient')
    def test_update_remove_member(self, SwaggerClient):
        CorpMember.objects.create(character_id='2', character_name='old test character', corpstats=self.corpstats)
        SwaggerClient.from_spec.return_value.Character.get_characters_character_id.return_value.result.return_value = {'corporation_id': 2}
        SwaggerClient.from_spec.return_value.Corporation.get_corporations_corporation_id_members.return_value.result.return_value = [1]
        SwaggerClient.from_spec.return_value.Universe.post_universe_names.return_value.result.return_value = [{'id': 1, 'name': 'test character'}]
        self.corpstats.update()
        self.assertFalse(CorpMember.objects.filter(character_id='2', corpstats=self.corpstats).exists())

    @mock.patch('allianceauth.corputils.models.notify')
    @mock.patch('esi.clients.SwaggerClient')
    def test_update_deleted_token(self, SwaggerClient, notify):
        SwaggerClient.from_spec.return_value.Character.get_characters_character_id.return_value.result.return_value = {'corporation_id': 2}
        SwaggerClient.from_spec.return_value.Corporation.get_corporations_corporation_id_members.return_value.result.side_effect = TokenError()
        self.corpstats.update()
        self.assertFalse(CorpStats.objects.filter(corp=self.corp).exists())
        self.assertTrue(notify.called)

    @mock.patch('allianceauth.corputils.models.notify')
    @mock.patch('esi.clients.SwaggerClient')
    def test_update_http_forbidden(self, SwaggerClient, notify):
        SwaggerClient.from_spec.return_value.Character.get_characters_character_id.return_value.result.return_value = {'corporation_id': 2}
        SwaggerClient.from_spec.return_value.Corporation.get_corporations_corporation_id_members.return_value.result.side_effect = HTTPForbidden(mock.Mock())
        self.corpstats.update()
        self.assertFalse(CorpStats.objects.filter(corp=self.corp).exists())
        self.assertTrue(notify.called)

    @mock.patch('allianceauth.corputils.models.notify')
    @mock.patch('esi.clients.SwaggerClient')
    def test_update_token_character_corp_changed(self, SwaggerClient, notify):
        SwaggerClient.from_spec.return_value.Character.get_characters_character_id.return_value.result.return_value = {'corporation_id': 3}
        self.corpstats.update()
        self.assertFalse(CorpStats.objects.filter(corp=self.corp).exists())
        self.assertTrue(notify.called)


class CorpStatsPropertiesTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test')
        AuthUtils.add_main_character(cls.user, 'test character', '1', corp_id='2', corp_name='test_corp', corp_ticker='TEST', alliance_id='3', alliance_name='TEST')
        cls.user.profile.refresh_from_db()
        cls.token = Token.objects.create(user=cls.user, access_token='a', character_id='1', character_name='test character', character_owner_hash='z')
        cls.corp = EveCorporationInfo.objects.create(corporation_id='2', corporation_name='test corp', corporation_ticker='TEST', member_count=1)
        cls.corpstats = CorpStats.objects.create(token=cls.token, corp=cls.corp)
        cls.character = EveCharacter.objects.create(character_name='another test character', character_id='4', corporation_id='2', corporation_name='test corp', corporation_ticker='TEST')

    def test_member_count(self):
        member = CorpMember.objects.create(corpstats=self.corpstats, character_id='1', character_name='test character')
        self.assertEqual(self.corpstats.member_count, 1)
        member.delete()
        self.assertEqual(self.corpstats.member_count, 0)

    def test_user_count(self):
        AuthUtils.disconnect_signals()
        co = CharacterOwnership.objects.create(character=self.character, user=self.user, owner_hash='a')
        AuthUtils.connect_signals()
        CorpMember.objects.create(corpstats=self.corpstats, character_id='4', character_name='test character')
        self.assertEqual(self.corpstats.user_count, 1)
        co.delete()
        self.assertEqual(self.corpstats.user_count, 0)

    def test_registered_members(self):
        AuthUtils.disconnect_signals()
        co = CharacterOwnership.objects.create(character=self.character, user=self.user, owner_hash='a')
        AuthUtils.connect_signals()
        member = CorpMember.objects.create(corpstats=self.corpstats, character_id='4', character_name='test character')
        self.assertIn(member, self.corpstats.registered_members)
        self.assertEqual(self.corpstats.registered_member_count, 1)

        co.delete()
        self.assertNotIn(member, self.corpstats.registered_members)
        self.assertEqual(self.corpstats.registered_member_count, 0)

    def test_unregistered_members(self):
        member = CorpMember.objects.create(corpstats=self.corpstats, character_id='4', character_name='test character')
        self.corpstats.refresh_from_db()
        self.assertIn(member, self.corpstats.unregistered_members)
        self.assertEqual(self.corpstats.unregistered_member_count, 1)

        AuthUtils.disconnect_signals()
        CharacterOwnership.objects.create(character=self.character, user=self.user, owner_hash='a')
        AuthUtils.connect_signals()
        self.assertNotIn(member, self.corpstats.unregistered_members)
        self.assertEqual(self.corpstats.unregistered_member_count, 0)

    def test_mains(self):
        # test when is a main
        member = CorpMember.objects.create(corpstats=self.corpstats, character_id='1', character_name='test character')
        self.assertIn(member, self.corpstats.mains)
        self.assertEqual(self.corpstats.main_count, 1)

        # test when is an alt
        old_main = self.user.profile.main_character
        character = EveCharacter.objects.create(character_name='other character', character_id=10, corporation_name='test corp', corporation_id='2', corporation_ticker='TEST')
        AuthUtils.disconnect_signals()
        co = CharacterOwnership.objects.create(character=character, user=self.user, owner_hash='b')
        self.user.profile.main_character = character
        self.user.profile.save()
        AuthUtils.connect_signals()
        self.assertNotIn(member, self.corpstats.mains)
        self.assertEqual(self.corpstats.main_count, 0)

        # test when no ownership
        co.delete()
        self.assertNotIn(member, self.corpstats.mains)
        self.assertEqual(self.corpstats.main_count, 0)

        # transaction won't roll this back
        AuthUtils.disconnect_signals()
        self.user.profile.main_character = old_main
        self.user.profile.save()
        AuthUtils.connect_signals()

    def test_logos(self):
        self.assertEqual(self.corpstats.corp_logo(size=128), 'https://image.eveonline.com/Corporation/2_128.png')
        self.assertEqual(self.corpstats.alliance_logo(size=128), 'https://image.eveonline.com/Alliance/1_128.png')

        alliance = EveAllianceInfo.objects.create(alliance_name='test alliance', alliance_id='3', alliance_ticker='TEST', executor_corp_id='2')
        self.corp.alliance = alliance
        self.corp.save()
        self.assertEqual(self.corpstats.alliance_logo(size=128), 'https://image.eveonline.com/Alliance/3_128.png')
        alliance.delete()


class CorpMemberTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('test')
        AuthUtils.add_main_character(cls.user, 'test character', '1', corp_id='2', corp_name='test_corp', corp_ticker='TEST', alliance_id='3', alliance_name='TEST')
        cls.user.profile.refresh_from_db()
        cls.token = Token.objects.create(user=cls.user, access_token='a', character_id='1', character_name='test character', character_owner_hash='a')
        cls.corp = EveCorporationInfo.objects.create(corporation_id='2', corporation_name='test corp', corporation_ticker='TEST', member_count=1)
        cls.corpstats = CorpStats.objects.create(token=cls.token, corp=cls.corp)
        cls.member = CorpMember.objects.create(corpstats=cls.corpstats, character_id='2', character_name='other test character')

    def test_character(self):
        self.assertIsNone(self.member.character)
        character = EveCharacter.objects.create(character_id='2', character_name='other test character', corporation_id='2', corporation_name='test corp', corporation_ticker='TEST')
        self.assertEqual(self.member.character, character)

    def test_main_character(self):
        AuthUtils.disconnect_signals()

        # test when member.character is None
        self.assertIsNone(self.member.main_character)

        # test when member.character is not None but also not a main
        character = EveCharacter.objects.create(character_id='2', character_name='other test character', corporation_id='2', corporation_name='test corp', corporation_ticker='TEST')
        CharacterOwnership.objects.create(character=character, user=self.user, owner_hash='b')
        self.member.refresh_from_db()
        self.assertNotEqual(self.member.main_character, self.member.character)
        self.assertEquals(self.member.main_character, self.user.profile.main_character)

        # test when is main
        old_main = self.user.profile.main_character
        self.user.profile.main_character = character
        self.user.profile.save()
        self.member.refresh_from_db()
        self.assertEqual(self.member.main_character, self.member.character)
        self.assertEqual(self.user.profile.main_character, self.member.main_character)

        # transaction won't roll this back
        self.user.profile.main_character = old_main
        self.user.profile.save()
        AuthUtils.connect_signals()

    def test_alts(self):
        self.assertListEqual(self.member.alts, [])

        character = EveCharacter.objects.create(character_id='2', character_name='other test character', corporation_id='2', corporation_name='test corp', corporation_ticker='TEST')
        CharacterOwnership.objects.create(character=character, user=self.user, owner_hash='b')
        self.assertIn(character, self.member.alts)

    def test_registered(self):
        self.assertFalse(self.member.registered)
        AuthUtils.disconnect_signals()
        character = EveCharacter.objects.create(character_id='2', character_name='other test character', corporation_id='2', corporation_name='test corp', corporation_ticker='TEST')
        CharacterOwnership.objects.create(character=character, user=self.user, owner_hash='b')
        self.assertTrue(self.member.registered)
        AuthUtils.connect_signals()

    def test_portrait_url(self):
        self.assertEquals(self.member.portrait_url(size=32), 'https://image.eveonline.com/Character/2_32.jpg')
        self.assertEquals(self.member.portrait_url(size=32), self.member.portrait_url_32)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-18 13:20
from __future__ import unicode_literals

from django.db import migrations, models


def get_duplicates(items):
    return set([item for item in items if items.count(item) > 1])


def enforce_unique_characters(apps, schema_editor):
    EveCharacter = apps.get_model('eveonline', 'EveCharacter')

    ids = [c.character_id for c in EveCharacter.objects.all()]
    duplicates = get_duplicates(ids)
    for c_id in duplicates:
        dupes = EveCharacter.objects.filter(character_id=c_id)
        dupes.exclude(pk=dupes[0].pk).delete()

    names = [c.character_name for c in EveCharacter.objects.all()]
    duplicates = get_duplicates(names)
    for name in duplicates:
        dupes = EveCharacter.objects.filter(character_name=name)
        dupes.exclude(pk=dupes[0].pk).delete()


def enforce_unique_corporations(apps, schema_editor):
    EveCorporationInfo = apps.get_model('eveonline', 'EveCorporationInfo')

    ids = [c.corporation_id for c in EveCorporationInfo.objects.all()]
    duplicates = get_duplicates(ids)
    for c_id in duplicates:
        dupes = EveCorporationInfo.objects.filter(corporation_id=c_id)
        dupes.exclude(pk=dupes[0].pk).delete()

    names = [c.corporation_name for c in EveCorporationInfo.objects.all()]
    duplicates = get_duplicates(names)
    for name in duplicates:
        dupes = EveCorporationInfo.objects.filter(character_name=name)
        dupes.exclude(pk=dupes[0].pk).delete()


def enforce_unique_alliances(apps, schema_editor):
    EveAllianceInfo = apps.get_model('eveonline', 'EveAllianceInfo')
    EveCorporationInfo = apps.get_model('eveonline', 'EveCorporationInfo')

    ids = [a.alliance_id for a in EveAllianceInfo.objects.all()]
    duplicates = get_duplicates(ids)
    for a_id in duplicates:
        dupes = EveAllianceInfo.objects.filter(alliance_id=a_id)
        to_be_kept = dupes[0]
        EveCorporationInfo.objects.filter(alliance__pk__in=[a.pk for a in dupes.exclude(pk=to_be_kept.pk)]).update(
            alliance=to_be_kept.pk)
        dupes.exclude(pk=to_be_kept.pk).delete()

    names = [a.alliance_name for a in EveAllianceInfo.objects.all()]
    duplicates = get_duplicates(names)
    for name in duplicates:
        dupes = EveAllianceInfo.objects.filter(alliance_name=name)
        to_be_kept = dupes[0]
        EveCorporationInfo.objects.filter(alliance__in=[a.pk for a in dupes.exclude(pk=to_be_kept.pk)]).update(
            alliance=to_be_kept.pk)
        dupes.exclude(pk=to_be_kept.pk).delete()


def enforce_unique_apis(apps, schema_editor):
    EveApiKeyPair = apps.get_model('eveonline', 'EveApiKeyPair')

    ids = [api.api_id for api in EveApiKeyPair.objects.all()]
    duplicates = get_duplicates(ids)
    for api_id in duplicates:
        dupes = EveApiKeyPair.objects.filter(api_id=api_id)
        dupes.exclude(pk=dupes[0].pk).delete()


class Migration(migrations.Migration):
    dependencies = [
        ('eveonline', '0006_allow_null_evecharacter_alliance'),
    ]

    operations = [
        migrations.RunPython(enforce_unique_characters, migrations.RunPython.noop),
        migrations.RunPython(enforce_unique_corporations, migrations.RunPython.noop),
        migrations.RunPython(enforce_unique_alliances, migrations.RunPython.noop),
        migrations.RunPython(enforce_unique_apis, migrations.RunPython.noop),
        migrations.AlterField(
            model_name='evecharacter',
            name='character_id',
            field=models.CharField(max_length=254, unique=True),
        ),
        migrations.AlterField(
            model_name='evecharacter',
            name='character_name',
            field=models.CharField(max_length=254, unique=True),
        ),
        migrations.AlterField(
            model_name='evecorporationinfo',
            name='corporation_id',
            field=models.CharField(max_length=254, unique=True),
        ),
        migrations.AlterField(
            model_name='evecorporationinfo',
            name='corporation_name',
            field=models.CharField(max_length=254, unique=True),
        ),
        migrations.AlterField(
            model_name='eveallianceinfo',
            name='alliance_id',
            field=models.CharField(max_length=254, unique=True),
        ),
        migrations.AlterField(
            model_name='eveallianceinfo',
            name='alliance_name',
            field=models.CharField(max_length=254, unique=True),
        ),
        migrations.AlterField(
            model_name='eveapikeypair',
            name='api_id',
            field=models.CharField(max_length=254, unique=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-09-02 06:07
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('services', '0002_auto_20161016_0135'),
    ]

    operations = [
        migrations.DeleteModel(
            name='GroupCache',
        ),
    ]



import logging

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from celery import shared_task

from allianceauth.notifications import notify
from allianceauth.services.hooks import NameFormatter
from allianceauth.services.tasks import QueueOnce
from .manager import DiscourseManager
from .models import DiscourseUser

logger = logging.getLogger(__name__)


class DiscourseTasks:
    def __init__(self):
        pass

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user) and user.discourse.enabled:
            logger.debug("User %s has a Discourse account. Disabling login." % user)
            if DiscourseManager.disable_user(user):
                user.discourse.delete()
                if notify_user:
                    notify(user, 'Discourse Account Disabled', level='danger')
                return True
        return False

    @staticmethod
    def has_account(user):
        """
        Check if the user has a discourse account
        :param user: django.contrib.auth.models.User
        :return: bool
        """
        try:
            return user.discourse.enabled
        except ObjectDoesNotExist:
            return False

    @staticmethod
    @shared_task(bind=True, name='discourse.update_groups', base=QueueOnce)
    def update_groups(self, pk):
        user = User.objects.get(pk=pk)
        logger.debug("Updating discourse groups for user %s" % user)
        try:
            DiscourseManager.update_groups(user)
        except:
            logger.warn("Discourse group sync failed for %s, retrying in 10 mins" % user)
            raise self.retry(countdown=60 * 10)
        logger.debug("Updated user %s discourse groups." % user)

    @staticmethod
    @shared_task(name='discourse.update_all_groups')
    def update_all_groups():
        logger.debug("Updating ALL discourse groups")
        for discourse_user in DiscourseUser.objects.filter(enabled=True):
            DiscourseTasks.update_groups.delay(discourse_user.user.pk)

    @staticmethod
    def get_username(user):
        from .auth_hooks import DiscourseService
        return NameFormatter(DiscourseService(), user).format_name()



from django.conf.urls import url

from . import views

app_name = 'permissions_tool'

urlpatterns = [
    url(r'^overview/$', views.permissions_overview, name='overview'),
    url(r'^audit/(?P<app_label>[\w\-_]+)/(?P<model>[\w\-_]+)/(?P<codename>[\w\-_]+)/$', views.permissions_audit,
        name='audit'),
]



import logging

from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import permission_required
from django.contrib.auth.decorators import user_passes_test
from django.shortcuts import render, get_object_or_404, redirect, Http404
from django.db.models import Q
from .models import Application
from .models import ApplicationComment
from .models import ApplicationForm
from .models import ApplicationResponse
from allianceauth.notifications import notify

from .forms import HRApplicationCommentForm
from .forms import HRApplicationSearchForm

logger = logging.getLogger(__name__)


def create_application_test(user):
    return bool(user.profile.main_character)


@login_required
def hr_application_management_view(request):
    logger.debug("hr_application_management_view called by user %s" % request.user)
    corp_applications = []
    finished_corp_applications = []
    main_char = request.user.profile.main_character

    base_app_query = Application.objects.select_related('user', 'form', 'form__corp')
    if request.user.is_superuser:
        corp_applications = base_app_query.filter(approved=None).order_by('-created')
        finished_corp_applications = base_app_query.exclude(approved=None).order_by('-created')
    elif request.user.has_perm('auth.human_resources') and main_char:
        if ApplicationForm.objects.filter(corp__corporation_id=main_char.corporation_id).exists():
            app_form = ApplicationForm.objects.get(corp__corporation_id=main_char.corporation_id)
            corp_applications = base_app_query.filter(form=app_form).filter(approved=None).order_by('-created')
            finished_corp_applications = base_app_query.filter(form=app_form).filter(
                approved__in=[True, False]).order_by('-created')
    logger.debug("Retrieved %s personal, %s corp applications for %s" % (
        len(request.user.applications.all()), len(corp_applications), request.user))
    context = {
        'personal_apps': request.user.applications.all(),
        'applications': corp_applications,
        'finished_applications': finished_corp_applications,
        'search_form': HRApplicationSearchForm(),
        'create': create_application_test(request.user)
    }
    return render(request, 'hrapplications/management.html', context=context)


@login_required
@user_passes_test(create_application_test)
def hr_application_create_view(request, form_id=None):
    if form_id:
        app_form = get_object_or_404(ApplicationForm, id=form_id)
        if request.method == "POST":
            if Application.objects.filter(user=request.user).filter(form=app_form).exists():
                logger.warn("User %s attempting to duplicate application to %s" % (request.user, app_form.corp))
            else:
                application = Application(user=request.user, form=app_form)
                application.save()
                for question in app_form.questions.all():
                    response = ApplicationResponse(question=question, application=application)
                    response.answer = "\n".join(request.POST.getlist(str(question.pk),
                                                       ""))
                    response.save()
                logger.info("%s created %s" % (request.user, application))
            return redirect('hrapplications:personal_view', application.id)
        else:
            questions = app_form.questions.all()
            return render(request, 'hrapplications/create.html',
                          context={'questions': questions, 'corp': app_form.corp})
    else:
        choices = []
        for app_form in ApplicationForm.objects.all():
            if not Application.objects.filter(user=request.user).filter(form=app_form).exists():
                choices.append((app_form.id, app_form.corp.corporation_name))
        return render(request, 'hrapplications/corpchoice.html', context={'choices': choices})


@login_required
def hr_application_personal_view(request, app_id):
    logger.debug("hr_application_personal_view called by user %s for app id %s" % (request.user, app_id))
    app = get_object_or_404(Application, pk=app_id)
    if app.user == request.user:
        context = {
            'app': app,
            'responses': ApplicationResponse.objects.filter(application=app),
            'buttons': False,
            'comments': ApplicationComment.objects.filter(application=app),
            'comment_form': HRApplicationCommentForm(),
        }
        return render(request, 'hrapplications/view.html', context=context)
    else:
        logger.warn("User %s not authorized to view %s" % (request.user, app))
        return redirect('hrapplications:personal_view')


@login_required
def hr_application_personal_removal(request, app_id):
    logger.debug("hr_application_personal_removal called by user %s for app id %s" % (request.user, app_id))
    app = get_object_or_404(Application, pk=app_id)
    if app.user == request.user:
        if app.approved is None:
            logger.info("User %s deleting %s" % (request.user, app))
            app.delete()
        else:
            logger.warn("User %s attempting to delete reviewed app %s" % (request.user, app))
    else:
        logger.warn("User %s not authorized to delete %s" % (request.user, app))
    return redirect('hrapplications:index')


@login_required
@permission_required('auth.human_resources')
def hr_application_view(request, app_id):
    logger.debug("hr_application_view called by user %s for app id %s" % (request.user, app_id))
    try:
        app = Application.objects.prefetch_related('responses', 'comments', 'comments__user').get(pk=app_id)
    except Application.DoesNotExist:
        raise Http404
    if request.method == 'POST':
        if request.user.has_perm('hrapplications.add_applicationcomment'):
            form = HRApplicationCommentForm(request.POST)
            logger.debug("Request type POST contains form valid: %s" % form.is_valid())
            if form.is_valid():
                comment = ApplicationComment()
                comment.application = app
                comment.user = request.user
                comment.text = form.cleaned_data['comment']
                comment.save()
                logger.info("Saved comment by user %s to %s" % (request.user, app))
                return redirect('hrapplications:view', app_id)
        else:
            logger.warn("User %s does not have permission to add ApplicationComments" % request.user)
            return redirect('hrapplications:view', app_id)
    else:
        logger.debug("Returning blank HRApplication comment form.")
        form = HRApplicationCommentForm()
    context = {
        'app': app,
        'responses': app.responses.all(),
        'buttons': True,
        'comments': app.comments.all(),
        'comment_form': form,
    }
    return render(request, 'hrapplications/view.html', context=context)


@login_required
@permission_required('auth.human_resources')
@permission_required('hrapplications.delete_application')
def hr_application_remove(request, app_id):
    logger.debug("hr_application_remove called by user %s for app id %s" % (request.user, app_id))
    app = get_object_or_404(Application, pk=app_id)
    logger.info("User %s deleting %s" % (request.user, app))
    app.delete()
    notify(app.user, "Application Deleted", message="Your application to %s was deleted." % app.form.corp)
    return redirect('hrapplications:index')


@login_required
@permission_required('auth.human_resources')
@permission_required('hrapplications.approve_application')
def hr_application_approve(request, app_id):
    logger.debug("hr_application_approve called by user %s for app id %s" % (request.user, app_id))
    app = get_object_or_404(Application, pk=app_id)
    if request.user.is_superuser or request.user == app.reviewer:
        logger.info("User %s approving %s" % (request.user, app))
        app.approved = True
        app.save()
        notify(app.user, "Application Accepted", message="Your application to %s has been approved." % app.form.corp,
               level="success")
    else:
        logger.warn("User %s not authorized to approve %s" % (request.user, app))
    return redirect('hrapplications:index')


@login_required
@permission_required('auth.human_resources')
@permission_required('hrapplications.reject_application')
def hr_application_reject(request, app_id):
    logger.debug("hr_application_reject called by user %s for app id %s" % (request.user, app_id))
    app = get_object_or_404(Application, pk=app_id)
    if request.user.is_superuser or request.user == app.reviewer:
        logger.info("User %s rejecting %s" % (request.user, app))
        app.approved = False
        app.save()
        notify(app.user, "Application Rejected", message="Your application to %s has been rejected." % app.form.corp,
               level="danger")
    else:
        logger.warn("User %s not authorized to reject %s" % (request.user, app))
    return redirect('hrapplications:index')


@login_required
@permission_required('auth.human_resources')
def hr_application_search(request):
    logger.debug("hr_application_search called by user %s" % request.user)
    if request.method == 'POST':
        form = HRApplicationSearchForm(request.POST)
        logger.debug("Request type POST contains form valid: %s" % form.is_valid())
        if form.is_valid():
            searchstring = form.cleaned_data['search_string'].lower()
            applications = set([])
            logger.debug("Searching for application with character name %s for user %s" % (searchstring, request.user))
            app_list = Application.objects.all()
            if not request.user.is_superuser:
                try:
                    app_list = app_list.filter(
                        form__corp__corporation_id=request.user.profile.main_character.corporation_id)
                except AttributeError:
                    logger.warn(
                        "User %s missing main character model: unable to filter applications to search" % request.user)

            applications = app_list.filter(
                Q(user__profile__main_character__character_name__icontains=searchstring) |
                Q(user__profile__main_character__corporation_name__icontains=searchstring) |
                Q(user__profile__main_character__alliance_name__icontains=searchstring) |
                Q(user__character_ownerships__character__character_name__icontains=searchstring) |
                Q(user__character_ownerships__character__corporation_name__icontains=searchstring) |
                Q(user__character_ownerships__character__alliance_name__icontains=searchstring) |
                Q(user__username__icontains=searchstring)
            )

            context = {'applications': applications, 'search_form': HRApplicationSearchForm()}

            return render(request, 'hrapplications/searchview.html', context=context)
        else:
            logger.debug("Form invalid - returning for user %s to retry." % request.user)
            context = {'applications': None, 'search_form': form}
            return render(request, 'hrapplications/searchview.html', context=context)

    else:
        logger.debug("Returning empty search form for user %s" % request.user)
        return redirect("hrapplications:view")


@login_required
@permission_required('auth.human_resources')
def hr_application_mark_in_progress(request, app_id):
    logger.debug("hr_application_mark_in_progress called by user %s for app id %s" % (request.user, app_id))
    app = get_object_or_404(Application, pk=app_id)
    if not app.reviewer:
        logger.info("User %s marking %s in progress" % (request.user, app))
        app.reviewer = request.user
        app.reviewer_character = request.user.profile.main_character
        app.save()
        notify(app.user, "Application In Progress",
               message="Your application to %s is being reviewed by %s" % (app.form.corp, app.reviewer_str))
    else:
        logger.warn(
            "User %s unable to mark %s in progress: already being reviewed by %s" % (request.user, app, app.reviewer))
    return redirect("hrapplications:view", app_id)



from django.contrib.auth.models import Group
from django.contrib.auth.models import User
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from allianceauth.authentication.models import State


class GroupRequest(models.Model):
    status = models.CharField(max_length=254)
    leave_request = models.BooleanField(default=0)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)

    @property
    def main_char(self):
        """
        Legacy property for main character
        :return: self.users main character
        """
        return self.user.profile.main_character

    def __str__(self):
        return self.user.username + ":" + self.group.name


class RequestLog(models.Model):
    request_type = models.NullBooleanField(default=0)
    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    request_info = models.CharField(max_length=254)
    action = models.BooleanField(default=0)
    request_actor = models.ForeignKey(User, on_delete=models.CASCADE)

    def requestor(self):
        return self.request_info.split(":")[0]

    def type_to_str(self):
        if self.request_type is None:
            return "Removed"
        elif self.request_type is True:
            return "Leave"
        elif self.request_type is False:
            return "Join"

    def action_to_str(self):
        if self.action is True:
            return "Accept"
        elif self.action is False:
            return "Reject"

    def req_char(self):
        usr = self.requestor()
        user = User.objects.get(username=usr)
        return user.profile.main_character



class AuthGroup(models.Model):
    """
    Extends Django Group model with a one-to-one field
    Attributes are accessible via group as if they were in the model
    e.g. group.authgroup.internal

    Logic:
    Internal - not requestable by users, at all. Covers Corp_, Alliance_, Members etc groups.
    Groups are internal by default

    Public - Other options are respected, but any user will be able to become and remain a member, even if they
    have no API etc entered. Auth will not manage these groups automatically so user removal is up to
    group managers/leaders.

    Not Internal and:
        Hidden - users cannot view, can request if they have the direct link.
        Not Hidden - Users can view and request the group
        Open - Users are automatically accepted into the group
        Not Open - Users requests must be approved before they are added to the group
    """
    group = models.OneToOneField(Group, on_delete=models.CASCADE, primary_key=True)

    internal = models.BooleanField(default=True,
                                   help_text="Internal group, users cannot see, join or request to join this group.<br>"
                                             "Used for groups such as Members, Corp_*, Alliance_* etc.<br>"
                                             "<b>Overrides Hidden and Open options when selected.</b>")
    hidden = models.BooleanField(default=True,
                                 help_text="Group is hidden from users but can still join with the correct link.")
    open = models.BooleanField(default=False,
                               help_text="Group is open and users will be automatically added upon request. <br>"
                                         "If the group is not open users will need their request manually approved.")
    public = models.BooleanField(default=False,
                                 help_text="Group is public. Any registered user is able to join this group, with "
                                           "visibility based on the other options set for this group.<br> Auth will "
                                           "not remove users from this group automatically when they are no longer "
                                           "authenticated.")
    # Group leaders have management access to this group
    group_leaders = models.ManyToManyField(User, related_name='leads_groups', blank=True,
                                           help_text="Group leaders can process group requests for this group "
                                                     "specifically. Use the auth.group_management permission to allow "
                                                     "a user to manage all groups.")

    states = models.ManyToManyField(State, related_name='valid_states', blank=True,
                                    help_text="States listed here will have the ability to join this group provided "
                                              "they have the proper permissions.")

    description = models.CharField(max_length=512, blank=True, help_text="Description of the group shown to users.")

    def __str__(self):
        return self.group.name

    class Meta:
        permissions = (
            ("request_groups", u"Can request non-public groups"),
        )
        default_permissions = tuple()


@receiver(post_save, sender=Group)
def create_auth_group(sender, instance, created, **kwargs):
    """
    Creates the AuthGroup model when a group is created
    """
    if created:
        AuthGroup.objects.create(group=instance)


@receiver(post_save, sender=Group)
def save_auth_group(sender, instance, **kwargs):
    """
    Ensures AuthGroup model is saved automatically
    """
    instance.authgroup.save()



from django.conf.urls import url

from . import views
app_name = 'fleetactivitytracking'

urlpatterns = [
    # FleetActivityTracking (FAT)
    url(r'^$', views.fatlink_view, name='view'),
    url(r'^statistics/$', views.fatlink_statistics_view, name='statistics'),
    url(r'^statistics/corp/(\w+)$', views.fatlink_statistics_corp_view,
        name='statistics_corp'),
    url(r'^statistics/corp/(?P<corpid>\w+)/(?P<year>[0-9]+)/(?P<month>[0-9]+)/',
        views.fatlink_statistics_corp_view,
        name='statistics_corp_month'),
    url(r'^statistics/(?P<year>[0-9]+)/(?P<month>[0-9]+)/$', views.fatlink_statistics_view,
        name='statistics_month'),
    url(r'^user/statistics/$', views.fatlink_personal_statistics_view,
        name='personal_statistics'),
    url(r'^user/statistics/(?P<year>[0-9]+)/$', views.fatlink_personal_statistics_view,
        name='personal_statistics_year'),
    url(r'^user/statistics/(?P<year>[0-9]+)/(?P<month>[0-9]+)/$',
        views.fatlink_monthly_personal_statistics_view,
        name='personal_statistics_month'),
    url(r'^user/(?P<char_id>[0-9]+)/statistics/(?P<year>[0-9]+)/(?P<month>[0-9]+)/$',
        views.fatlink_monthly_personal_statistics_view,
        name='user_statistics_month'),
    url(r'^create/$', views.create_fatlink_view, name='create'),
    url(r'^modify/(?P<fat_hash>[a-zA-Z0-9_-]+)/$', views.modify_fatlink_view, name='modify'),
    url(r'^link/(?P<fat_hash>[a-zA-Z0-9]+)/$', views.click_fatlink_view, name='click'),
]



import logging

from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import Permission, User
from django.db.models import Count
from django.shortcuts import render, Http404


logger = logging.getLogger(__name__)


@login_required
@permission_required('permissions_tool.audit_permissions')
def permissions_overview(request):
    logger.debug("permissions_overview called by user %s" % request.user)
    perms = Permission.objects.select_related('content_type').all()\
        .annotate(Count('user', distinct=True))\
        .annotate(Count('group', distinct=True)) \
        .annotate(Count('group__user', distinct=True)) \
        .annotate(Count('state', distinct=True))\
        .annotate(Count('state__userprofile', distinct=True))

    get_all = True if request.GET.get('all', 'no') == 'yes' else False

    context = {'permissions': []}
    for perm in perms:
        this_perm = {
            'users': perm.user__count,
            'groups': perm.group__count,
            'group_users': perm.group__user__count,
            'states': perm.state__count,
            'state_users': perm.state__userprofile__count,
            'permission': perm,
        }

        if get_all or sum([this_perm['users'], this_perm['groups'], this_perm['states']]) > 0:
            # Only add if we're getting everything or one of the objects has this permission
            context['permissions'].append(this_perm)

    return render(request, 'permissions_tool/overview.html', context=context)


@login_required
@permission_required('permissions_tool.audit_permissions')
def permissions_audit(request, app_label, model, codename):
    logger.debug("permissions_audit called by user {} on {}:{}:{}".format(request.user, app_label, model, codename))
    try:
        perm = Permission.objects\
            .prefetch_related('group_set', 'user_set', 'state_set',
                              'state_set__userprofile_set', 'group_set__user_set', 'state_set__userprofile_set__user')\
            .get(content_type__app_label=app_label, content_type__model=model, codename=codename)
    except Permission.DoesNotExist:
        raise Http404

    context = {'permission': {
        'permission': perm,
        'users': perm.user_set.all(),
        'groups': perm.group_set.all(),
        'states': perm.state_set.all(),
        'group_users': [group.user_set.all() for group in perm.group_set.all()],
        'state_users': [state.userprofile_set.all() for state in perm.state_set.all()],
        }
    }

    return render(request, 'permissions_tool/audit.html', context=context)



# Generated by Django 2.0.6 on 2018-07-11 00:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0016_ownershiprecord'),
        ('groupmanagement', '0009_requestlog'),
    ]

    operations = [
        migrations.AddField(
            model_name='authgroup',
            name='states',
            field=models.ManyToManyField(blank=True, help_text='States listed here will have the ability to join this group provided they have the proper permissions.', related_name='valid_states', to='authentication.State'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-01-15 07:06
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='SeatUser',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='seat', serialize=False, to=settings.AUTH_USER_MODEL)),
                ('username', models.CharField(max_length=254)),
            ],
        ),
    ]



from unittest import mock

from django.test import TestCase, RequestFactory
from django import urls
from django.contrib.auth.models import User, Group, Permission
from django.core.exceptions import ObjectDoesNotExist

from allianceauth.tests.auth_utils import AuthUtils

from .auth_hooks import MarketService
from .models import MarketUser
from .tasks import MarketTasks

MODULE_PATH = 'allianceauth.services.modules.market'
DEFAULT_AUTH_GROUP = 'Member'


def add_permissions():
    permission = Permission.objects.get(codename='access_market')
    members = Group.objects.get_or_create(name=DEFAULT_AUTH_GROUP)[0]
    AuthUtils.add_permissions_to_groups([permission], [members])


class MarketHooksTestCase(TestCase):
    def setUp(self):
        self.member = 'member_user'
        member = AuthUtils.create_member(self.member)
        MarketUser.objects.create(user=member, username=self.member)
        self.none_user = 'none_user'
        none_user = AuthUtils.create_user(self.none_user)
        self.service = MarketService
        add_permissions()

    def test_has_account(self):
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)
        self.assertTrue(MarketTasks.has_account(member))
        self.assertFalse(MarketTasks.has_account(none_user))

    def test_service_enabled(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)

        self.assertTrue(service.service_active_for_user(member))
        self.assertFalse(service.service_active_for_user(none_user))

    @mock.patch(MODULE_PATH + '.tasks.MarketManager')
    def test_delete_user(self, manager):
        member = User.objects.get(username=self.member)

        service = self.service()
        result = service.delete_user(member)

        self.assertTrue(result)
        self.assertTrue(manager.disable_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            market_user = User.objects.get(username=self.member).market

    @mock.patch(MODULE_PATH + '.tasks.MarketManager')
    def test_validate_user(self, manager):
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.member)
        service.validate_user(member)
        self.assertTrue(User.objects.get(username=self.member).market)

        # Test none user is deleted
        none_user = User.objects.get(username=self.none_user)
        MarketUser.objects.create(user=none_user, username='abc123')
        service.validate_user(none_user)
        self.assertTrue(manager.disable_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            none_svc = User.objects.get(username=self.none_user).market

    def test_render_services_ctrl(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        request = RequestFactory().get('/services/')
        request.user = member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn(urls.reverse('evernusmarket:set_password'), response)
        self.assertIn(urls.reverse('evernusmarket:reset_password'), response)
        self.assertIn(urls.reverse('evernusmarket:deactivate'), response)

        # Test register becomes available
        member.market.delete()
        member = User.objects.get(username=self.member)
        request.user = member
        response = service.render_services_ctrl(request)
        self.assertIn(urls.reverse('evernusmarket:activate'), response)


class MarketViewsTestCase(TestCase):
    def setUp(self):
        self.member = AuthUtils.create_member('auth_member')
        self.member.email = 'auth_member@example.com'
        self.member.save()
        AuthUtils.add_main_character(self.member, 'auth_member', '12345', corp_id='111', corp_name='Test Corporation')
        add_permissions()

    def login(self):
        self.client.force_login(self.member)

    @mock.patch(MODULE_PATH + '.views.MarketManager')
    def test_activate(self, manager):
        self.login()
        expected_username = 'auth_member'
        expected_password = 'password'
        expected_id = '1234'

        manager.add_user.return_value = (expected_username, expected_password, expected_id)

        response = self.client.get(urls.reverse('evernusmarket:activate'), follow=False)

        self.assertTrue(manager.add_user.called)
        args, kwargs = manager.add_user.call_args
        self.assertEqual(args[0], expected_username)
        self.assertEqual(args[1], self.member.email)

        self.assertTemplateUsed(response, 'services/service_credentials.html')
        self.assertContains(response, expected_username)
        self.assertContains(response, expected_password)

    @mock.patch(MODULE_PATH + '.tasks.MarketManager')
    def test_deactivate(self, manager):
        self.login()
        MarketUser.objects.create(user=self.member, username='12345')
        manager.disable_user.return_value = True

        response = self.client.get(urls.reverse('evernusmarket:deactivate'))

        self.assertTrue(manager.disable_user.called)
        self.assertRedirects(response, expected_url=urls.reverse('services:services'), target_status_code=200)
        with self.assertRaises(ObjectDoesNotExist):
            market_user = User.objects.get(pk=self.member.pk).market

    @mock.patch(MODULE_PATH + '.views.MarketManager')
    def test_set_password(self, manager):
        self.login()
        MarketUser.objects.create(user=self.member, username='12345')
        expected_password = 'password'
        manager.update_user_password.return_value = expected_password

        response = self.client.post(urls.reverse('evernusmarket:set_password'), data={'password': expected_password})

        self.assertTrue(manager.update_custom_password.called)
        args, kwargs = manager.update_custom_password.call_args
        self.assertEqual(args[1], expected_password)
        self.assertRedirects(response, expected_url=urls.reverse('services:services'), target_status_code=200)

    @mock.patch(MODULE_PATH + '.views.MarketManager')
    def test_reset_password(self, manager):
        self.login()
        MarketUser.objects.create(user=self.member, username='12345')

        response = self.client.get(urls.reverse('evernusmarket:reset_password'))

        self.assertTrue(manager.update_user_password.called)
        self.assertTemplateUsed(response, 'services/service_credentials.html')


class MarketManagerTestCase(TestCase):
    def setUp(self):
        from .manager import MarketManager
        self.manager = MarketManager

    def test_generate_random_password(self):
        password = self.manager._MarketManager__generate_random_pass()

        self.assertEqual(len(password), 16)
        self.assertIsInstance(password, type(''))

    def test_gen_pwhash(self):
        pwhash = self.manager._gen_pwhash('test')
        salt = self.manager._get_salt(pwhash)

        self.assertIsInstance(pwhash, str)
        self.assertGreaterEqual(len(pwhash), 59)
        self.assertIsInstance(salt, str)
        self.assertEqual(len(salt), 22)



import logging

from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from allianceauth.notifications import notify
from celery import shared_task
from requests.exceptions import HTTPError
from allianceauth.services.hooks import NameFormatter
from .manager import DiscordOAuthManager, DiscordApiBackoff
from .models import DiscordUser
from allianceauth.services.tasks import QueueOnce

logger = logging.getLogger(__name__)


class DiscordTasks:
    def __init__(self):
        pass

    @classmethod
    def add_user(cls, user, code):
        groups = DiscordTasks.get_groups(user)
        nickname = None
        if settings.DISCORD_SYNC_NAMES:
            nickname = DiscordTasks.get_nickname(user)
        user_id = DiscordOAuthManager.add_user(code, groups, nickname=nickname)
        if user_id:
            discord_user = DiscordUser()
            discord_user.user = user
            discord_user.uid = user_id
            discord_user.save()
            return True
        return False

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user):
            logger.debug("User %s has discord account %s. Deleting." % (user, user.discord.uid))
            if DiscordOAuthManager.delete_user(user.discord.uid):
                user.discord.delete()
                if notify_user:
                    notify(user, 'Discord Account Disabled', level='danger')
                return True
        return False

    @classmethod
    def has_account(cls, user):
        """
        Check if the user has an account (has a DiscordUser record)
        :param user: django.contrib.auth.models.User
        :return: bool
        """
        try:
            user.discord
        except ObjectDoesNotExist:
            return False
        else:
            return True

    @staticmethod
    @shared_task(bind=True, name='discord.update_groups', base=QueueOnce)
    def update_groups(self, pk):
        user = User.objects.get(pk=pk)
        logger.debug("Updating discord groups for user %s" % user)
        if DiscordTasks.has_account(user):
            groups = DiscordTasks.get_groups(user)
            logger.debug("Updating user %s discord groups to %s" % (user, groups))
            try:
                DiscordOAuthManager.update_groups(user.discord.uid, groups)
            except DiscordApiBackoff as bo:
                logger.info("Discord group sync API back off for %s, "
                            "retrying in %s seconds" % (user, bo.retry_after_seconds))
                raise self.retry(countdown=bo.retry_after_seconds)
            except HTTPError as e:
                if e.response.status_code == 404:
                    try:
                        if e.response.json()['code'] == 10007:
                            # user has left the server
                            DiscordTasks.delete_user(user)
                            return
                    finally:
                        raise e
            except Exception as e:
                if self:
                    logger.exception("Discord group sync failed for %s, retrying in 10 mins" % user)
                    raise self.retry(countdown=60 * 10)
                else:
                    # Rethrow
                    raise e
            logger.debug("Updated user %s discord groups." % user)
        else:
            logger.debug("User does not have a discord account, skipping")

    @staticmethod
    @shared_task(name='discord.update_all_groups')
    def update_all_groups():
        logger.debug("Updating ALL discord groups")
        for discord_user in DiscordUser.objects.exclude(uid__exact=''):
            DiscordTasks.update_groups.delay(discord_user.user.pk)

    @staticmethod
    @shared_task(bind=True, name='discord.update_nickname', base=QueueOnce)
    def update_nickname(self, pk):
        user = User.objects.get(pk=pk)
        logger.debug("Updating discord nickname for user %s" % user)
        if DiscordTasks.has_account(user):
            if user.profile.main_character:
                character = user.profile.main_character
                logger.debug("Updating user %s discord nickname to %s" % (user, character.character_name))
                try:
                    DiscordOAuthManager.update_nickname(user.discord.uid, DiscordTasks.get_nickname(user))
                except DiscordApiBackoff as bo:
                    logger.info("Discord nickname update API back off for %s, "
                                "retrying in %s seconds" % (user, bo.retry_after_seconds))
                    raise self.retry(countdown=bo.retry_after_seconds)
                except Exception as e:
                    if self:
                        logger.exception("Discord nickname sync failed for %s, retrying in 10 mins" % user)
                        raise self.retry(countdown=60 * 10)
                    else:
                        # Rethrow
                        raise e
                logger.debug("Updated user %s discord nickname." % user)
            else:
                logger.debug("User %s does not have a main character" % user)
        else:
            logger.debug("User %s does not have a discord account" % user)

    @staticmethod
    @shared_task(name='discord.update_all_nicknames')
    def update_all_nicknames():
        logger.debug("Updating ALL discord nicknames")
        for discord_user in DiscordUser.objects.exclude(uid__exact=''):
            DiscordTasks.update_nickname.delay(discord_user.user.pk)

    @classmethod
    def disable(cls):
        DiscordUser.objects.all().delete()

    @staticmethod
    def get_nickname(user):
        from .auth_hooks import DiscordService
        return NameFormatter(DiscordService(), user).format_name()

    @staticmethod
    def get_groups(user):
        return [g.name for g in user.groups.all()] + [user.profile.state.name]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-10 05:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0005_delete_perms'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='discourse_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='forum_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='ipboard_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='ips4_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='jabber_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='market_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='mumble_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='smf_password',
        ),
        migrations.RemoveField(
            model_name='authservicesinfo',
            name='xenforo_password',
        ),
    ]



from django.conf.urls import url, include

from . import views

app_name='seat'

module_urls = [
    # SeAT Service Control
    url(r'^activate/$', views.activate_seat, name='activate'),
    url(r'^deactivate/$', views.deactivate_seat, name='deactivate'),
    url(r'^reset_password/$', views.reset_seat_password, name='reset_password'),
    url(r'^set_password/$', views.set_seat_password, name='set_password'),
]

urlpatterns = [
    url(r'^seat/', include((module_urls, app_name), namespace=app_name)),
]



import logging
import datetime
from django import forms
from django.utils import timezone
from django.core.validators import MaxValueValidator, MinValueValidator
from django.utils.translation import ugettext_lazy as _

from .models import Timer

logger = logging.getLogger(__name__)


class TimerForm(forms.ModelForm):
    class Meta:
        model = Timer
        fields = ('details', 'system', 'planet_moon', 'structure', 'objective', 'important', 'corp_timer')

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        if 'instance' in kwargs and kwargs['instance'] is not None:
            # Do conversion from db datetime to days/hours/minutes
            # for appropriate fields
            current_time = timezone.now()
            td = kwargs['instance'].eve_time - current_time
            initial = kwargs.pop('initial', dict())
            if 'days_left' not in initial:
                initial.update({'days_left': td.days})
            if 'hours_left' not in initial:
                initial.update({'hours_left': td.seconds // 3600})
            if 'minutes_left' not in initial:
                initial.update({'minutes_left': td.seconds // 60 % 60})
            kwargs.update({'initial': initial})
        super(TimerForm, self).__init__(*args, **kwargs)

    structure_choices = [('POCO', 'POCO'),
                         ('I-HUB', 'I-HUB'),
                         ('POS[S]', 'POS[S]'),
                         ('POS[M]', 'POS[M]'),
                         ('POS[L]', 'POS[L]'),
                         ('Citadel[M]', 'Citadel[M]'),
                         ('Citadel[L]', 'Citadel[L]'),
                         ('Citadel[XL]', 'Citadel[XL]'),
                         ('Engineering Complex[M]', 'Engineering Complex[M]'),
                         ('Engineering Complex[L]', 'Engineering Complex[L]'),
                         ('Engineering Complex[XL]', 'Engineering Complex[XL]'),
                         ('Refinery[M]', 'Refinery[M]'),
                         ('Refinery[L]', 'Refinery[L]'),
                         ('Station', 'Station'),
                         ('TCU', 'TCU'),
                         ('Moon Mining Cycle', 'Moon Mining Cycle'),
                         (_('Other'), _('Other'))]
    objective_choices = [('Friendly', _('Friendly')),
                         ('Hostile', _('Hostile')),
                         ('Neutral', _('Neutral'))]

    details = forms.CharField(max_length=254, required=True, label=_('Details'))
    system = forms.CharField(max_length=254, required=True, label=_("System"))
    planet_moon = forms.CharField(max_length=254, label=_("Planet/Moon"), required=False, initial="")
    structure = forms.ChoiceField(choices=structure_choices, required=True, label=_("Structure Type"))
    objective = forms.ChoiceField(choices=objective_choices, required=True, label=_("Objective"))
    days_left = forms.IntegerField(required=True, label=_("Days Remaining"), validators=[MinValueValidator(0)])
    hours_left = forms.IntegerField(required=True, label=_("Hours Remaining"),
                                    validators=[MinValueValidator(0), MaxValueValidator(23)])
    minutes_left = forms.IntegerField(required=True, label=_("Minutes Remaining"),
                                      validators=[MinValueValidator(0), MaxValueValidator(59)])
    important = forms.BooleanField(label=_("Important"), required=False)
    corp_timer = forms.BooleanField(label=_("Corp-Restricted"), required=False)

    def save(self, commit=True):
        timer = super(TimerForm, self).save(commit=False)

        # Get character
        character = self.user.profile.main_character
        corporation = character.corporation
        logger.debug("Determined timer save request on behalf "
                     "of character {} corporation {}".format(character, corporation))
        # calculate future time
        future_time = datetime.timedelta(days=self.cleaned_data['days_left'], hours=self.cleaned_data['hours_left'],
                                         minutes=self.cleaned_data['minutes_left'])
        current_time = timezone.now()
        eve_time = current_time + future_time
        logger.debug(
            "Determined timer eve time is %s - current time %s, adding %s" % (eve_time, current_time, future_time))

        timer.eve_time = eve_time
        timer.eve_character = character
        timer.eve_corp = corporation
        timer.user = self.user
        if commit:
            timer.save()
        return timer



from django.conf.urls import url
from . import views

app_name = 'notifications'
# Notifications
urlpatterns = [
    url(r'^remove_notifications/(\w+)/$', views.remove_notification, name='remove'),
    url(r'^notifications/mark_all_read/$', views.mark_all_read, name='mark_all_read'),
    url(r'^notifications/delete_all_read/$', views.delete_all_read, name='delete_all_read'),
    url(r'^notifications/$', views.notification_list, name='list'),
    url(r'^notifications/(\w+)/$', views.notification_view, name='view'),
]



from django.contrib import admin
from .models import DiscordUser


class DiscordUserAdmin(admin.ModelAdmin):
        list_display = ('user', 'uid')
        search_fields = ('user__username', 'uid')

admin.site.register(DiscordUser, DiscordUserAdmin)



from unittest import mock

import requests_mock
import json
import datetime

from django.test import TestCase
from django.utils.timezone import make_aware, utc

from allianceauth.fleetup.managers import FleetUpManager


class FleetupManagerTestCase(TestCase):
    def setUp(self):
        pass

    def test__request_cache_key(self):

        cache_key = FleetUpManager._request_cache_key('testurl')

        self.assertEqual('FLEETUP_ENDPOINT_a39562b6ef5b858220be13d2adb61d3f10cf8d61',
                         cache_key)

    @mock.patch('allianceauth.fleetup.managers.cache')
    @requests_mock.Mocker()
    def test_get_endpoint(self, cache, m):
        url = "http://example.com/test/endpoint/"
        json_data = {'data': "123456", 'CachedUntilUTC': '/Date(1493896236163)/', 'Success': True}
        m.register_uri('GET', url,
                       text=json.dumps(json_data))

        cache.get.return_value = None  # No cached value

        # Act
        result = FleetUpManager.get_endpoint(url)

        # Assert
        self.assertTrue(cache.get.called)
        self.assertTrue(cache.set.called)
        args, kwargs = cache.set.call_args
        self.assertDictEqual(json_data, args[1])

        self.assertDictEqual(json_data, result)

    @mock.patch('allianceauth.fleetup.managers.cache')
    @requests_mock.Mocker()
    def test_get_endpoint_error(self, cache, m):
        url = "http://example.com/test/endpoint/"
        json_data = {'data': [], 'Success': False}
        m.register_uri('GET', url,
                       text=json.dumps(json_data),
                       status_code=400)

        cache.get.return_value = None  # No cached value

        # Act
        result = FleetUpManager.get_endpoint(url)

        # Assert
        self.assertTrue(cache.get.called)
        self.assertFalse(cache.set.called)
        self.assertIsNone(result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_members(self, get_endpoint):

        get_endpoint.return_value = {"Data": [
            {
                'UserId': 1234,
                'EveCharName': 'test_name',
                'EveCharId': 5678,
                'Corporation': 'test_corporation',
            }
        ]}

        # Act
        result = FleetUpManager.get_fleetup_members()

        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0],
                         FleetUpManager.BASE_URL + '/GroupCharacters/' +
                         FleetUpManager.GROUP_ID)
        expected_result = {
            1234: {
                'user_id': 1234,
                'char_name': 'test_name',
                'char_id': 5678,
                'corporation': 'test_corporation',
            }
        }
        self.assertDictEqual(expected_result, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_members()

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': []}

        # Act
        result = FleetUpManager.get_fleetup_members()

        # Assert
        self.assertDictEqual({}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_operations(self, get_endpoint):

        get_endpoint.return_value = {"Data": [
            {
                'Subject': 'test_operation',
                'StartString': '2017-05-06 11:11:11',
                'EndString': '2017-05-06 12:12:12',
                'OperationId': 1234,
                'Location': 'Jita',
                'LocationInfo': '4-4',
                'Details': 'This is a test operation',
                'Url': 'http://example.com/1234',
                'Doctrines': 'Foxcats',
                'Organizer': 'Example FC'
            }
        ]}

        # Act
        result = FleetUpManager.get_fleetup_operations()
        self.maxDiff = None
        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0],
                         FleetUpManager.BASE_URL + '/Operations/' +
                         FleetUpManager.GROUP_ID)
        expected_result = {
            '2017-05-06 11:11:11': {
                'subject': 'test_operation',
                'start': make_aware(datetime.datetime(2017, 5, 6, 11, 11, 11), utc),
                'end': make_aware(datetime.datetime(2017, 5, 6, 12, 12, 12), utc),
                'operation_id': 1234,
                'location': 'Jita',
                'location_info': '4-4',
                'details': 'This is a test operation',
                'url': 'http://example.com/1234',
                'doctrine': 'Foxcats',
                'organizer': 'Example FC'
            }
        }
        self.assertDictEqual(expected_result, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_operations()

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': []}

        # Act
        result = FleetUpManager.get_fleetup_operations()

        # Assert
        self.assertDictEqual({}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_timers(self, get_endpoint):

        get_endpoint.return_value = {"Data": [
            {
                'ExpiresString': '2017-05-06 11:11:11',
                'SolarSystem': 'Jita',
                'Planet': '4',
                'Moon': '4',
                'Owner': 'Caldari Navy',
                'Type': 'Caldari Station',
                'TimerType': 'Armor',
                'Notes': 'Burn Jita?'
            }
        ]}

        # Act
        result = FleetUpManager.get_fleetup_timers()

        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0],
                         FleetUpManager.BASE_URL + '/Timers/' +
                         FleetUpManager.GROUP_ID)
        expected_result = {
            '2017-05-06 11:11:11': {
                'expires': make_aware(datetime.datetime(2017, 5, 6, 11, 11, 11), utc),
                'solarsystem': 'Jita',
                'planet': '4',
                'moon': '4',
                'owner': 'Caldari Navy',
                'type': 'Caldari Station',
                'timer_type': 'Armor',
                'notes': 'Burn Jita?'
            }
        }
        self.assertDictEqual(expected_result, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_timers()

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': []}

        # Act
        result = FleetUpManager.get_fleetup_timers()

        # Assert
        self.assertDictEqual({}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_doctrines(self, get_endpoint):

        get_endpoint.return_value = {"Data": [
            {
                'TestData': True
            }
        ]}

        # Act
        result = FleetUpManager.get_fleetup_doctrines()

        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0],
                         FleetUpManager.BASE_URL + '/Doctrines/' +
                         FleetUpManager.GROUP_ID)
        expected_result = {
            'fleetup_doctrines': [{
                'TestData': True
            }]
        }
        self.assertDictEqual(expected_result, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_doctrines()

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': []}

        # Act
        result = FleetUpManager.get_fleetup_doctrines()

        # Assert
        self.assertDictEqual({"fleetup_doctrines": []}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_doctrine(self, get_endpoint):

        get_endpoint.return_value = {"Data": [
            {
                'TestData': True
            }
        ]}

        # Act
        result = FleetUpManager.get_fleetup_doctrine(1234)

        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0],
                         FleetUpManager.BASE_URL + '/DoctrineFittings/1234')
        expected_result = {
            'fitting_doctrine': {'Data': [{
                'TestData': True
            }]}
        }
        self.assertDictEqual(expected_result, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_doctrine(1234)

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': []}

        # Act
        result = FleetUpManager.get_fleetup_doctrine(1234)

        # Assert
        self.assertDictEqual({"fitting_doctrine": {'Data': []}}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_fittings(self, get_endpoint):

        get_endpoint.return_value = {"Data": [
            {
                'FittingId': 1234,
                'Name': 'Foxcat',
                'EveTypeId': 17726,
                'HullType': 'Battleship',
                'ShipType': 'Apocalypse Navy Issue',
                'EstPrice': 500000000,
                'Faction': 'Amarr',
                'Categories': ["Armor", "Laser"],
                'LastUpdatedString': '2017-05-06 11:11:11',
            }
        ]}

        # Act
        result = FleetUpManager.get_fleetup_fittings()

        # Asset
        self.assertTrue(get_endpoint.called)
        expected_result = {
            1234: {
                'fitting_id': 1234,
                'name': 'Foxcat',
                'icon_id': 17726,
                'hull': 'Battleship',
                'shiptype': 'Apocalypse Navy Issue',
                'estimated': 500000000,
                'faction': 'Amarr',
                'categories': ["Armor", "Laser"],
                'last_update': make_aware(datetime.datetime(2017, 5, 6, 11, 11, 11), utc)
            }
        }
        self.assertDictEqual(expected_result, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_fittings()

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': []}

        # Act
        result = FleetUpManager.get_fleetup_fittings()

        # Assert
        self.assertDictEqual({}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_fitting(self, get_endpoint):

        get_endpoint.return_value = {"Data":
            {
                'FittingData': [{}]
            }
        }

        # Act
        result = FleetUpManager.get_fleetup_fitting(1234)

        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0], FleetUpManager.BASE_URL + '/Fitting/1234')
        expected_result = {
            'fitting_data': {
                'FittingData': [{}]
            }
        }
        self.assertDictEqual(expected_result, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_fitting(1234)

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': {}}

        # Act
        result = FleetUpManager.get_fleetup_fitting(1234)

        # Assert
        self.assertDictEqual({"fitting_data": {}}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_doctrineid(self, get_endpoint):

        get_endpoint.return_value = {
            "Data": {
                'Doctrines': [{'DoctrineId': 4567}]
            }
        }

        # Act
        result = FleetUpManager.get_fleetup_doctrineid(1234)

        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0], FleetUpManager.BASE_URL + '/Fitting/1234')

        self.assertEqual(4567, result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_doctrineid(1234)

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': {}}

        # Act
        result = FleetUpManager.get_fleetup_doctrineid(1234)

        # Assert
        self.assertDictEqual({}, result)

    @mock.patch('allianceauth.fleetup.managers.FleetUpManager.get_endpoint')
    def test_get_fleetup_fitting_eft(self, get_endpoint):

        get_endpoint.return_value = {
            "Data": {
                'FittingData': '[Apocalypse Navy Issue, Foxcat]'
            }
        }

        # Act
        result = FleetUpManager.get_fleetup_fitting_eft(1234)

        # Asset
        self.assertTrue(get_endpoint.called)
        args, kwargs = get_endpoint.call_args
        self.assertEqual(args[0], FleetUpManager.BASE_URL + '/Fitting/1234/eft')

        self.assertDictEqual({"fitting_eft": '[Apocalypse Navy Issue, Foxcat]'},
                             result)

        # Test None response
        # Arrange
        get_endpoint.return_value = None

        # Act
        result = FleetUpManager.get_fleetup_fitting_eft(1234)

        # Assert
        self.assertIsNone(result)

        # Test Empty response
        # Arrange
        get_endpoint.return_value = {'Data': {}}

        # Act
        result = FleetUpManager.get_fleetup_fitting_eft(1234)

        # Assert
        self.assertDictEqual({"fitting_eft": {}}, result)



from django.conf.urls import include, url
from django.template.loader import render_to_string
from django.utils.functional import cached_property
from django.conf import settings
from string import Formatter

from allianceauth.hooks import get_hooks

from .models import NameFormatConfig


class ServicesHook:
    """
    Abstract base class for creating a compatible services
    hook. Decorate with @register('services_hook') to have the
    services module registered for callbacks. Must be in
    auth_hook(.py) sub module
    """
    def __init__(self):
        self.name = 'Undefined'
        self.urlpatterns = []
        self.service_ctrl_template = 'services/services_ctrl.html'
        self.access_perm = None

    @property
    def title(self):
        """
        A nicely formatted title of the service, for client facing
        display.
        :return: str
        """
        return self.name.title()

    def delete_user(self, user, notify_user=False):
        """
        Delete the users service account, optionally notify them
        that the service has been disabled
        :param user: Django.contrib.auth.models.User
        :param notify_user: Whether the service should sent a
        notification to the user about the disabling of their
        service account.
        :return: True if the service account has been disabled,
        or False if it doesnt exist.
        """
        pass

    def validate_user(self, user):
        pass

    def sync_nickname(self, user):
        """
        Sync the users nickname
        :param user: Django.contrib.auth.models.User
        :return: None
        """
        pass

    def update_groups(self, user):
        """
        Update the users group membership
        :param user: Django.contrib.auth.models.User
        :return: None
        """
        pass

    def update_all_groups(self):
        """
        Iterate through and update all users groups
        :return: None
        """
        pass

    def service_active_for_user(self, user):
        pass

    def show_service_ctrl(self, user):
        """
        Whether the service control should be displayed to the given user
        who has the given service state. Usually this function wont
        require overloading.
        :param user: django.contrib.auth.models.User
        :return: bool True if the service should be shown
        """
        return self.service_active_for_user(user) or user.is_superuser

    def render_services_ctrl(self, request):
        """
        Render the services control template row
        :param request:
        :return:
        """
        return ''

    def __str__(self):
        return self.name or 'Unknown Service Module'

    class Urls:
        def __init__(self):
            self.auth_activate = ''
            self.auth_set_password = ''
            self.auth_reset_password = ''
            self.auth_deactivate = ''

    @staticmethod
    def get_services():
        for fn in get_hooks('services_hook'):
            yield fn()


class MenuItemHook:
    def __init__(self, text, classes, url_name, order=None, navactive=list([])):
        self.text = text
        self.classes = classes
        self.url_name = url_name
        self.template = 'public/menuitem.html'
        self.order = order if order is not None else 9999
        navactive = navactive or []
        navactive.append(url_name)
        self.navactive = navactive

    def render(self, request):
        return render_to_string(self.template,
                                {'item': self},
                                request=request)


class UrlHook:
    def __init__(self, urls, namespace, base_url):
        self.include_pattern = url(base_url, include(urls, namespace=namespace))


class NameFormatter:
    DEFAULT_FORMAT = getattr(settings, "DEFAULT_SERVICE_NAME_FORMAT", '[{corp_ticker}] {character_name}')

    def __init__(self, service, user):
        """
        :param service: ServicesHook of the service to generate the name for.
        :param user: django.contrib.auth.models.User to format name for
        """
        self.service = service
        self.user = user

    def format_name(self):
        """
        :return: str Generated name
        """
        format_data = self.get_format_data()
        return Formatter().vformat(self.string_formatter, args=[], kwargs=format_data)

    def get_format_data(self):
        main_char = getattr(self.user.profile, 'main_character', None)

        format_data = {
            'character_name': getattr(main_char, 'character_name',
                                      self.user.username if self._default_to_username else None),
            'character_id': getattr(main_char, 'character_id', None),
            'corp_ticker': getattr(main_char, 'corporation_ticker', None),
            'corp_name': getattr(main_char, 'corporation_name', None),
            'corp_id': getattr(main_char, 'corporation_id', None),
            'alliance_name': getattr(main_char, 'alliance_name', None),
            'alliance_id': getattr(main_char, 'alliance_id', None),
            'alliance_ticker': getattr(main_char, 'alliance_ticker', None),
            'username': self.user.username,
        }

        format_data['alliance_or_corp_name'] = format_data['alliance_name'] or format_data['corp_name']
        format_data['alliance_or_corp_ticker'] = format_data['alliance_ticker'] or format_data['corp_ticker']
        return format_data

    @cached_property
    def formatter_config(self):
        format_config = NameFormatConfig.objects.filter(service_name=self.service.name,
                                                        states__pk=self.user.profile.state.pk)

        if format_config.exists():
            return format_config[0]
        return None

    @cached_property
    def string_formatter(self):
        """
        Try to get the config format first
        Then the service default
        Before finally defaulting to global default
        :return: str
        """
        return getattr(self.formatter_config, 'format', self.default_formatter)

    @cached_property
    def default_formatter(self):
        return getattr(self.service, 'name_format', self.DEFAULT_FORMAT)

    @cached_property
    def _default_to_username(self):
        """
        Default to a users username if they have no main character.
        Default is True
        :return: bool
        """
        return getattr(self.formatter_config, 'default_to_username', True)



from django.conf.urls import url, include

from . import views

app_name = 'mumble'

module_urls = [
    # Mumble service control
    url(r'^activate/$', views.CreateAccountMumbleView.as_view(), name='activate'),
    url(r'^deactivate/$', views.DeleteMumbleView.as_view(), name='deactivate'),
    url(r'^reset_password/$', views.ResetPasswordMumbleView.as_view(), name='reset_password'),
    url(r'^set_password/$', views.SetPasswordMumbleView.as_view(), name='set_password'),
]

urlpatterns = [
    url(r'^mumble/', include((module_urls, app_name), namespace=app_name))
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-22 23:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('optimer', '0002_auto_20170413_0442'),
    ]

    operations = [
        migrations.AlterField(
            model_name='optimer',
            name='doctrine',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='optimer',
            name='duration',
            field=models.CharField(default='', max_length=25),
        ),
        migrations.AlterField(
            model_name='optimer',
            name='fc',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='optimer',
            name='operation_name',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='optimer',
            name='system',
            field=models.CharField(default='', max_length=254),
        ),
    ]
